@@ -1,5 +1,5 @@
 /* FileChannelImpl.java -- 
-   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -175,7 +175,8 @@ public final class FileChannelImpl extends FileChannel
    */
   protected void finalize() throws IOException
   {
-    this.close();
+    if (fd != -1)
+      close();
   }
 
   public int read (ByteBuffer dst) throws IOException
@@ -300,8 +301,10 @@ public final class FileChannelImpl extends FileChannel
     else if (mode == MapMode.READ_WRITE || mode == MapMode.PRIVATE)
       {
 	nmode = mode == MapMode.READ_WRITE ? '+' : 'c';
-	if ((this.mode & (READ|WRITE)) != (READ|WRITE))
+	if ((this.mode & WRITE) != WRITE)
 	  throw new NonWritableChannelException();
+	if ((this.mode & READ) != READ)
+	  throw new NonReadableChannelException();
       }
     else
       throw new IllegalArgumentException ("mode: " + mode);
@@ -524,7 +527,7 @@ public final class FileChannelImpl extends FileChannel
     throws IOException
   {
     if (newPosition < 0)
-      throw new IllegalArgumentException ("newPostition: " + newPosition);
+      throw new IllegalArgumentException ("newPosition: " + newPosition);
 
     if (!isOpen ())
       throw new ClosedChannelException ();
