@@ -640,7 +640,7 @@ public class Tiger extends BaseHash
   {
     if (valid == null)
       {
-        valid = new Boolean(DIGEST0.equals(Util.toString(new Tiger().digest())));
+        valid = Boolean.valueOf(DIGEST0.equals(Util.toString(new Tiger().digest())));
       }
     return valid.booleanValue();
   }
