@@ -852,7 +852,7 @@ public final class Rijndael extends BaseCipher
           {
             result = testKat(KAT_KEY, KAT_CT);
           }
-        valid = new Boolean(result);
+        valid = Boolean.valueOf(result);
       }
     return valid.booleanValue();
   }
