@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package java.awt;
 
+import gnu.classpath.NotImplementedException;
+
 import java.awt.geom.AffineTransform;
 import java.awt.image.BufferedImage;
 import java.awt.image.ColorModel;
@@ -130,11 +132,10 @@ public abstract class GraphicsConfiguration
    * with the given transparency. Because the buffer is volatile, it
    * can be optimized by native graphics accelerators.
    *
-   * @param w the width of the buffer
-   * @param h the height of the buffer
+   * @param width the width of the buffer
+   * @param height the height of the buffer
    * @param transparency the transparency value for the buffer
    * @return the buffered image, or null if none is supported
-   * @throws AWTException if the capabilities cannot be met
    * @since 1.5
    */
   public abstract VolatileImage createCompatibleVolatileImage(int width,
@@ -217,6 +218,7 @@ public abstract class GraphicsConfiguration
    * @since 1.4
    */
   public BufferCapabilities getBufferCapabilities()
+    throws NotImplementedException
   {
     throw new Error("not implemented");
   }
@@ -228,6 +230,7 @@ public abstract class GraphicsConfiguration
    * @since 1.4
    */
   public ImageCapabilities getImageCapabilities()
+    throws NotImplementedException
   {
     throw new Error("not implemented");
   }
