@@ -38,7 +38,9 @@ exception statement from your version. */
 
 package java.awt;
 
+import java.awt.image.AreaAveragingScaleFilter;
 import java.awt.image.FilteredImageSource;
+import java.awt.image.ImageFilter;
 import java.awt.image.ImageObserver;
 import java.awt.image.ImageProducer;
 import java.awt.image.ReplicateScaleFilter;
@@ -141,7 +143,6 @@ public abstract class Image
    * This method is only valid for off-screen objects.
    *
    * @return a graphics context object for an off-screen object
-   * @see Graphics#getcreateImage(int, int)
    */
   public abstract Graphics getGraphics();
 
@@ -165,6 +166,8 @@ public abstract class Image
    * loading will be produced according to the hints of the algorithm
    * requested. If either the width or height is non-positive, it is adjusted
    * to preserve the original aspect ratio.
+   * If an illegal value of <code>flags</code> is passed,
+   * the default algorithm is used.
    *
    * @param width the width of the scaled image
    * @param height the height of the scaled image
@@ -179,20 +182,22 @@ public abstract class Image
    */
   public Image getScaledInstance(int width, int height, int flags)
   {
+    ImageFilter filter;
     switch (flags)
     {
+      case SCALE_AREA_AVERAGING:
+      case SCALE_SMOOTH:
+	filter = new AreaAveragingScaleFilter(width, height);
+	break;
       case SCALE_DEFAULT:
       case SCALE_FAST:
       case SCALE_REPLICATE:
-        ImageProducer producer =
-          new FilteredImageSource(this.getSource(),
-                                  new ReplicateScaleFilter(width, height));
-        return Toolkit.getDefaultToolkit().createImage(producer);
-      case SCALE_SMOOTH:
-      case SCALE_AREA_AVERAGING:
       default:
-        throw new Error("not implemented");
+	filter = new ReplicateScaleFilter(width, height);
     }
+
+    ImageProducer producer = new FilteredImageSource(getSource(), filter);
+    return Toolkit.getDefaultToolkit().createImage(producer);
   }
 
   /**
