@@ -1,5 +1,6 @@
 /* FilePermission.java --
-   Copyright (C) 1998, 2000, 2003, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 1998, 2000, 2003, 2004, 2005, 2006
+   Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +16,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -44,9 +45,6 @@ public final class FilePermission extends Permission implements Serializable
 {
   private static final long serialVersionUID = 7930732926638008763L;
 
-  private static final String CURRENT_DIRECTORY = 
-    System.getProperty("user.dir");
-
   private static final String ALL_FILES = "<<ALL FILES>>";
 
   private boolean readPerm = false;
@@ -213,10 +211,18 @@ public final class FilePermission extends Permission implements Serializable
     FilePermission fp = (FilePermission) p;
     String f2 = fp.getName();
 
-    if (f1.charAt(0) != File.separatorChar)
-      f1 = CURRENT_DIRECTORY + f1;
-    if (f2.charAt(0) != File.separatorChar)
-      f2 = CURRENT_DIRECTORY + f2;
+    if (f2.equals(ALL_FILES))
+      return false;
+
+    try
+      {
+	f1 = new File(f1).getCanonicalPath();
+	f2 = new File(f2).getCanonicalPath();
+      }
+    catch (IOException ioe)
+      {
+	return false;
+      }
 
     String sub1;
 
@@ -278,13 +284,13 @@ public final class FilePermission extends Permission implements Serializable
 	break;
       }
 
-    if (readPerm && ! fp.readPerm)
+    if (fp.readPerm && ! readPerm)
       return false;
-    if (writePerm && ! fp.writePerm)
+    if (fp.writePerm && ! writePerm)
       return false;
-    if (executePerm && ! fp.executePerm)
+    if (fp.executePerm && ! executePerm)
       return false;
-    if (deletePerm && ! fp.deletePerm)
+    if (fp.deletePerm && ! deletePerm)
       return false;
     
     return true;
