@@ -76,7 +76,8 @@ public class BigInteger extends Number implements Comparable
   private static final long serialVersionUID = -8287574255936472291L;
 
 
-  /** We pre-allocate integers in the range minFixNum..maxFixNum. */
+  /** We pre-allocate integers in the range minFixNum..maxFixNum. 
+   * Note that we must at least preallocate 0, 1, and 10.  */
   private static final int minFixNum = -100;
   private static final int maxFixNum = 1024;
   private static final int numFixNum = maxFixNum-minFixNum+1;
@@ -87,11 +88,23 @@ public class BigInteger extends Number implements Comparable
       smallFixNums[i] = new BigInteger(i + minFixNum);
   }
 
-  // JDK1.2
+  /**
+   * The constant zero as a BigInteger.
+   * @since 1.2
+   */
   public static final BigInteger ZERO = smallFixNums[-minFixNum];
 
-  // JDK1.2
+  /**
+   * The constant one as a BigInteger.
+   * @since 1.2
+   */
   public static final BigInteger ONE = smallFixNums[1 - minFixNum];
+  
+  /**
+   * The constant ten as a BigInteger.
+   * @since 1.5
+   */
+  public static final BigInteger TEN = smallFixNums[10 - minFixNum];
 
   /* Rounding modes: */
   private static final int FLOOR = 1;
@@ -343,9 +356,9 @@ public class BigInteger extends Number implements Comparable
 
   public int signum()
   {
-    int top = words == null ? ival : words[ival-1];
-    if (top == 0 && words == null)
+    if (ival == 0 && words == null)
       return 0;
+    int top = words == null ? ival : words[ival-1];
     return top < 0 ? -1 : 1;
   }
 
@@ -2214,17 +2227,25 @@ public class BigInteger extends Number implements Comparable
     throws IOException, ClassNotFoundException
   {
     s.defaultReadObject();
-    words = byteArrayToIntArray(magnitude, signum < 0 ? -1 : 0);
-    BigInteger result = make(words, words.length);
-    this.ival = result.ival;
-    this.words = result.words;
+    if (magnitude.length == 0 || signum == 0)
+      {
+        this.ival = 0;
+        this.words = null;
+      }
+    else
+      {
+        words = byteArrayToIntArray(magnitude, signum < 0 ? -1 : 0);
+        BigInteger result = make(words, words.length);
+        this.ival = result.ival;
+        this.words = result.words;        
+      }    
   }
 
   private void writeObject(ObjectOutputStream s)
     throws IOException, ClassNotFoundException
   {
     signum = signum();
-    magnitude = toByteArray();
+    magnitude = signum == 0 ? new byte[0] : toByteArray();
     s.defaultWriteObject();
   }
 }
