@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -248,6 +248,35 @@ public abstract class AccessibleContext
   public static final String ACCESSIBLE_HYPERTEXT_OFFSET
     = "AccessibleHypertextOffset";
 
+  /**
+   * Constant used when a component's bounds have changed.  The old and
+   * new bounds are given in the event.
+   * @since 1.5
+   */
+  public static final String ACCESSIBLE_COMPONENT_BOUNDS_CHANGED
+    = "accessibleComponentBoundsChanged";
+
+  /**
+   * Constant used when the state of child objects changes.  The old
+   * value in the event is always null, and the new value is the component
+   * whose children have changed.
+   * @since 1.5
+   */
+  public static final String ACCESSIBLE_INVALIDATE_CHILDREN
+    = "accessibleInvalidateChildren";
+
+  /**
+   * Constant used when the attributes of some text have changed.
+   * On insertion, the old value is null and the new value is an
+   * {@link AccessibleAttributeSequence} describing the insertion.
+   * On deletion, the old value is an {@link AccessibleAttributeSequence}
+   * and the new value is null.  For replacement, both the old
+   * and new values are {@link AccessibleAttributeSequence} objects.
+   * @since 1.5
+   */
+  public static final String ACCESSIBLE_TEXT_ATTRIBUTES_CHANGED
+    = "accessibleTextAttributesChanged";
+
   /**
    * The accessible parent of this object.
    *
@@ -540,14 +569,15 @@ public abstract class AccessibleContext
 
   /**
    * Get any supported accessible relation set. The default implementation
-   * returns null.
+   * returns an empty AccessibleRelationSet.
+   *
+   * @return the supported relation set, or <code>null</code>
    *
-   * @return the supported relation set, or null
    * @see AccessibleRelationSet
    */
   public AccessibleRelationSet getAccessibleRelationSet()
   {
-    return null;
+    return new AccessibleRelationSet();
   }
 
   /**
