@@ -79,6 +79,7 @@ public class ButtonGroup implements Serializable
    */
   public ButtonGroup()
   {
+    // Nothing to do here.
   }
 
   /**
@@ -89,7 +90,13 @@ public class ButtonGroup implements Serializable
   public void add(AbstractButton b)
   {
     b.getModel().setGroup(this);
-    buttons.addElement(b);
+    if (b.isSelected())
+      {
+        if (sel == null)
+          sel = b.getModel();
+        else
+          b.setSelected(false);
+      }    buttons.addElement(b);
   }
 
   /**
@@ -158,7 +165,7 @@ public class ButtonGroup implements Serializable
       {
         ButtonModel old = sel;
         sel = m;
-
+        
         if (old != null)
           old.setSelected(false);
         AbstractButton button = FindButton(old);
