@@ -1,5 +1,5 @@
 /* JComboBox.java --
-   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006,  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package javax.swing;
 
+import gnu.classpath.NotImplementedException;
+
 import java.awt.ItemSelectable;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
@@ -46,8 +48,6 @@ import java.awt.event.ItemListener;
 import java.awt.event.KeyEvent;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
-import java.io.IOException;
-import java.io.ObjectOutputStream;
 import java.util.Vector;
 
 import javax.accessibility.Accessible;
@@ -57,6 +57,7 @@ import javax.accessibility.AccessibleRole;
 import javax.accessibility.AccessibleSelection;
 import javax.swing.event.ListDataEvent;
 import javax.swing.event.ListDataListener;
+import javax.swing.event.PopupMenuEvent;
 import javax.swing.event.PopupMenuListener;
 import javax.swing.plaf.ComboBoxUI;
 
@@ -212,10 +213,6 @@ public class JComboBox extends JComponent implements ItemSelectable,
     this(new DefaultComboBoxModel());
   }
 
-  private void writeObject(ObjectOutputStream stream) throws IOException
-  {
-  }
-
   /**
    * This method returns true JComboBox is editable and false otherwise
    *
@@ -257,7 +254,6 @@ public class JComboBox extends JComponent implements ItemSelectable,
   public void updateUI()
   {
     setUI((ComboBoxUI) UIManager.getUI(this));
-    invalidate();
   }
 
   /**
@@ -310,7 +306,8 @@ public class JComboBox extends JComponent implements ItemSelectable,
     // Stores old data model for event notification.
     ComboBoxModel oldDataModel = dataModel;
     dataModel = newDataModel;
-
+    selectedItemReminder = newDataModel.getSelectedItem();
+    
     // Notifies the listeners of the model change.
     firePropertyChange("model", oldDataModel, dataModel);
   }
@@ -551,14 +548,37 @@ public class JComboBox extends JComponent implements ItemSelectable,
     return -1;
   }
 
+  /**
+   * Returns an object that is used as the display value when calculating the 
+   * preferred size for the combo box.  This value is, of course, never 
+   * displayed anywhere.
+   * 
+   * @return The prototype display value (possibly <code>null</code>).
+   * 
+   * @since 1.4
+   * @see #setPrototypeDisplayValue(Object)
+   */
   public Object getPrototypeDisplayValue()
   {
     return prototypeDisplayValue;
   }
 
-  public void setPrototypeDisplayValue(Object newPrototypeDisplayValue)
+  /**
+   * Sets the object that is assumed to be the displayed item when calculating
+   * the preferred size for the combo box.  A {@link PropertyChangeEvent} (with
+   * the name <code>prototypeDisplayValue</code>) is sent to all registered 
+   * listeners. 
+   * 
+   * @param value  the new value (<code>null</code> permitted).
+   * 
+   * @since 1.4
+   * @see #getPrototypeDisplayValue()
+   */
+  public void setPrototypeDisplayValue(Object value)
   {
-    prototypeDisplayValue = newPrototypeDisplayValue;
+    Object oldValue = prototypeDisplayValue;
+    prototypeDisplayValue = value;
+    firePropertyChange("prototypeDisplayValue", oldValue, value);
   }
 
   /**
@@ -819,6 +839,47 @@ public class JComboBox extends JComponent implements ItemSelectable,
                                             actionCommand));
   }
 
+  /**
+   * Fires a popupMenuCanceled() event to all <code>PopupMenuListeners</code>.
+   *
+   * Note: This method is intended for use by plaf classes only.
+   */
+  public void firePopupMenuCanceled()
+  {
+    PopupMenuListener[] listeners = getPopupMenuListeners();
+    PopupMenuEvent e = new PopupMenuEvent(this);
+    for(int i = 0; i < listeners.length; i++)
+      listeners[i].popupMenuCanceled(e);
+  }
+
+  /**
+   * Fires a popupMenuWillBecomeInvisible() event to all 
+   * <code>PopupMenuListeners</code>.
+   *
+   * Note: This method is intended for use by plaf classes only.
+   */
+  public void firePopupMenuWillBecomeInvisible()
+  {
+    PopupMenuListener[] listeners = getPopupMenuListeners();
+    PopupMenuEvent e = new PopupMenuEvent(this);
+    for(int i = 0; i < listeners.length; i++)
+      listeners[i].popupMenuWillBecomeInvisible(e);
+  }
+
+  /**
+   * Fires a popupMenuWillBecomeVisible() event to all 
+   * <code>PopupMenuListeners</code>.
+   *
+   * Note: This method is intended for use by plaf classes only.
+   */
+  public void firePopupMenuWillBecomeVisible()
+  {
+    PopupMenuListener[] listeners = getPopupMenuListeners();
+    PopupMenuEvent e = new PopupMenuEvent(this);
+    for(int i = 0; i < listeners.length; i++)
+      listeners[i].popupMenuWillBecomeVisible(e);
+  }
+
   /**
    * This method is invoked whenever selected item changes in the combo box's
    * data model. It fires ItemEvent and ActionEvent to all registered
@@ -836,8 +897,9 @@ public class JComboBox extends JComponent implements ItemSelectable,
 
     // Fire ItemEvent to indicate that new item is selected    
     Object newSelection = getSelectedItem();
-    fireItemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
-                                       newSelection, ItemEvent.SELECTED));
+    if (newSelection != null)
+      fireItemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
+                                         newSelection, ItemEvent.SELECTED));
 
     // Fire Action Event to JComboBox's registered listeners					 				 
     fireActionEvent();
@@ -866,7 +928,7 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void actionPerformed(ActionEvent e)
   {
-    setSelectedItem(((ComboBoxEditor) e.getSource()).getItem());
+    setSelectedItem(getEditor().getItem());
     setPopupVisible(false);
   }
 
@@ -883,8 +945,19 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public boolean selectWithKeyChar(char keyChar)
   {
-    // FIXME: Need to implement
-    return false;
+    if (keySelectionManager == null)
+      {
+        keySelectionManager = createDefaultKeySelectionManager();
+      }
+
+    int index = keySelectionManager.selectionForKey(keyChar, getModel());
+    boolean retVal = false;
+    if (index >= 0)
+      {
+        setSelectedIndex(index);
+        retVal = true;
+      }
+    return retVal;
   }
 
   /**
@@ -961,19 +1034,19 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void processKeyEvent(KeyEvent e)
   {
-  }
-
-  /**
-   * This method always returns false to indicate that JComboBox  itself is
-   * not focus traversable.
-   *
-   * @return false to indicate that JComboBox itself is not focus traversable.
-   *
-   * @deprecated
-   */
-  public boolean isFocusTraversable()
-  {
-    return false;
+    if (e.getKeyCode() == KeyEvent.VK_TAB)
+      setPopupVisible(false);
+    else if (keySelectionManager != null)
+      {
+        int i = keySelectionManager.selectionForKey(e.getKeyChar(),
+                                                    getModel());
+        if (i >= 0)
+          setSelectedIndex(i);
+        else
+          super.processKeyEvent(e);
+      }
+    else
+      super.processKeyEvent(e);
   }
 
   /**
@@ -983,6 +1056,7 @@ public class JComboBox extends JComponent implements ItemSelectable,
    */
   public void setKeySelectionManager(KeySelectionManager aManager)
   {
+    keySelectionManager = aManager;
   }
 
   /**
@@ -1028,15 +1102,33 @@ public class JComboBox extends JComponent implements ItemSelectable,
   }
 
   /**
-   * A string that describes this JComboBox. Normally only used for debugging.
+   * Returns an implementation-dependent string describing the attributes of
+   * this <code>JComboBox</code>.
    *
-   * @return A string describing this JComboBox
+   * @return A string describing the attributes of this <code>JComboBox</code>
+   *         (never <code>null</code>).
    */
   protected String paramString()
   {
-    return "JComboBox";
+    String superParamStr = super.paramString();
+    StringBuffer sb = new StringBuffer();
+    sb.append(",isEditable=").append(isEditable());
+    sb.append(",lightWeightPopupEnabled=").append(isLightWeightPopupEnabled());
+    sb.append(",maximumRowCount=").append(getMaximumRowCount());
+    
+    sb.append(",selectedItemReminder=");
+    if (selectedItemReminder != null)
+      sb.append(selectedItemReminder);
+    return superParamStr + sb.toString();
   }
 
+  /**
+   * Returns the object that provides accessibility features for this
+   * <code>JComboBox</code> component.
+   *
+   * @return The accessible context (an instance of 
+   *         {@link AccessibleJComboBox}).
+   */
   public AccessibleContext getAccessibleContext()
   {
     if (accessibleContext == null)
@@ -1145,79 +1237,107 @@ public class JComboBox extends JComponent implements ItemSelectable,
   {
     private static final long serialVersionUID = 8217828307256675666L;
 
-    protected AccessibleJComboBox()
+    /**
+     * @specnote This constructor was protected in 1.4, but made public
+     * in 1.5.
+     */
+    public AccessibleJComboBox()
     {
+      // Nothing to do here.
     }
 
     public int getAccessibleChildrenCount()
+      throws NotImplementedException
     {
       return 0;
     }
 
     public Accessible getAccessibleChild(int value0)
+      throws NotImplementedException
     {
       return null;
     }
 
     public AccessibleSelection getAccessibleSelection()
+      throws NotImplementedException
     {
       return null;
     }
 
     public Accessible getAccessibleSelection(int value0)
+      throws NotImplementedException
     {
       return null;
     }
 
     public boolean isAccessibleChildSelected(int value0)
+      throws NotImplementedException
     {
       return false;
     }
 
+    /**
+     * Returns the accessible role for the <code>JComboBox</code> component.
+     *
+     * @return {@link AccessibleRole#COMBO_BOX}.
+     */
     public AccessibleRole getAccessibleRole()
     {
       return AccessibleRole.COMBO_BOX;
     }
 
     public AccessibleAction getAccessibleAction()
+      throws NotImplementedException
     {
       return null;
     }
 
     public String getAccessibleActionDescription(int value0)
+      throws NotImplementedException
     {
       return null;
     }
 
     public int getAccessibleActionCount()
+      throws NotImplementedException
     {
       return 0;
     }
 
     public boolean doAccessibleAction(int value0)
+      throws NotImplementedException
     {
       return false;
     }
 
     public int getAccessibleSelectionCount()
+      throws NotImplementedException
     {
       return 0;
     }
 
     public void addAccessibleSelection(int value0)
+      throws NotImplementedException
     {
+      // TODO: Implement this properly.
     }
 
     public void removeAccessibleSelection(int value0)
+      throws NotImplementedException
     {
+      // TODO: Implement this properly.
     }
 
     public void clearAccessibleSelection()
+      throws NotImplementedException
     {
+      // TODO: Implement this properly.
     }
 
     public void selectAllAccessibleSelection()
+      throws NotImplementedException
     {
+      // TODO: Implement this properly.
     }
   }
 }
