@@ -37,10 +37,11 @@ exception statement from your version. */
 
 package javax.swing;
 
+import gnu.classpath.NotImplementedException;
+
 import java.awt.Component;
 import java.awt.Frame;
 import java.awt.HeadlessException;
-import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.beans.PropertyChangeEvent;
@@ -50,7 +51,6 @@ import java.util.ArrayList;
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
 import javax.accessibility.AccessibleRole;
-import javax.swing.JDialog;
 import javax.swing.filechooser.FileFilter;
 import javax.swing.filechooser.FileSystemView;
 import javax.swing.filechooser.FileView;
@@ -492,6 +492,7 @@ public class JFileChooser extends JComponent implements Accessible
    * @param b DOCUMENT ME!
    */
   public void setDragEnabled(boolean b)
+    throws NotImplementedException
   {
     // FIXME: Implement
   }
@@ -502,6 +503,7 @@ public class JFileChooser extends JComponent implements Accessible
    * @return DOCUMENT ME!
    */
   public boolean getDragEnabled()
+    throws NotImplementedException
   {
     // FIXME: Implement
     return false;
@@ -658,8 +660,7 @@ public class JFileChooser extends JComponent implements Accessible
 
     retval = ERROR_OPTION;
 
-    Insets i = d.getInsets();
-    d.setSize(500 + i.top + i.bottom, d.getPreferredSize().height);
+    d.pack();
     d.show();
     return retval;
   }
@@ -683,8 +684,7 @@ public class JFileChooser extends JComponent implements Accessible
 
     retval = ERROR_OPTION;
 
-    Insets i = d.getInsets();
-    d.setSize(500 + i.top + i.bottom, d.getPreferredSize().height);
+    d.pack();
     d.show();
     return retval;
   }
@@ -710,8 +710,7 @@ public class JFileChooser extends JComponent implements Accessible
 
     retval = ERROR_OPTION;
 
-    Insets i = d.getInsets();
-    d.setSize(500 + i.top + i.bottom, d.getPreferredSize().height);
+    d.pack();
     d.show();
     return retval;
   }
@@ -729,7 +728,7 @@ public class JFileChooser extends JComponent implements Accessible
   {
     Frame toUse = (Frame) SwingUtilities.getAncestorOfClass(Frame.class, parent);
     if (toUse == null)
-      toUse = SwingUtilities.getOwnerFrame();
+      toUse = (Frame) SwingUtilities.getOwnerFrame(null);
 
     JDialog dialog = new JDialog(toUse);
     setSelectedFile(null);
@@ -1492,7 +1491,6 @@ public class JFileChooser extends JComponent implements Accessible
   public void updateUI()
   {
     setUI((FileChooserUI) UIManager.getUI(this));
-    revalidate();
   }
 
   /**
@@ -1532,7 +1530,9 @@ public class JFileChooser extends JComponent implements Accessible
    */
   public AccessibleContext getAccessibleContext()
   {
-    return new AccessibleJFileChooser();
+    if (accessibleContext == null)
+      accessibleContext = new AccessibleJFileChooser();
+    return accessibleContext;
   }
 
   /**
