@@ -1,5 +1,5 @@
 /* JOptionPane.java
-   Copyright (C) 2004 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -39,7 +39,6 @@ exception statement from your version. */
 package javax.swing;
 
 import java.awt.Component;
-import java.awt.Dimension;
 import java.awt.Frame;
 
 import javax.accessibility.Accessible;
@@ -60,26 +59,30 @@ public class JOptionPane extends JComponent implements Accessible
   /**
    * DOCUMENT ME!
    */
+  // FIXME: This inner class is a complete stub and needs to be implemented
+  // properly.
   protected class AccessibleJOptionPane extends JComponent.AccessibleJComponent
   {
     /** DOCUMENT ME! */
     private static final long serialVersionUID = 686071432213084821L;
-
+    
     /**
      * Creates a new AccessibleJOptionPane object.
      */
     protected AccessibleJOptionPane()
     {
+      // Nothing to do here.
     }
 
     /**
-     * DOCUMENT ME!
+     * Returns the accessible role of this object, which is always
+     * {@link AccessibleRole#OPTION_PANE}.
      *
-     * @return DOCUMENT ME!
+     * @return the accessible role of this object
      */
     public AccessibleRole getAccessibleRole()
     {
-      return null;
+      return AccessibleRole.OPTION_PANE;
     }
   }
 
@@ -195,7 +198,7 @@ public class JOptionPane extends JComponent implements Accessible
   public static final String WANTS_INPUT_PROPERTY = "wantsInput";
 
   /** The value returned when the inputValue is uninitialized. */
-  public static Object UNINITIALIZED_VALUE = "uninitializedValue";
+  public static final Object UNINITIALIZED_VALUE = "uninitializedValue";
 
   /** The icon displayed in the dialog/internal frame. */
   protected Icon icon;
@@ -234,7 +237,7 @@ public class JOptionPane extends JComponent implements Accessible
   protected boolean wantsInput;
 
   /** The common frame used when no parent is provided. */
-  private static Frame privFrame = SwingUtilities.getOwnerFrame();
+  private static Frame privFrame = (Frame) SwingUtilities.getOwnerFrame(null);
 
   /**
    * Creates a new JOptionPane object using a message of "JOptionPane
@@ -343,8 +346,6 @@ public class JOptionPane extends JComponent implements Accessible
     setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
 
     updateUI();
-    invalidate();
-    repaint();
   }
 
   /**
@@ -369,17 +370,12 @@ public class JOptionPane extends JComponent implements Accessible
     inputValue = UNINITIALIZED_VALUE;
     value = UNINITIALIZED_VALUE;
 
-    // FIXME: This dialog should be centered on the parent
-    // or at the center of the screen (if the parent is null)
-    // Need getGraphicsConfiguration to return non-null in
-    // order for that to work so we know how large the 
-    // screen is.
     dialog.getContentPane().add(this);
     dialog.setModal(true);
     dialog.setResizable(false);
-    dialog.invalidate();
-    dialog.repaint();
-
+    dialog.pack();
+    dialog.setLocationRelativeTo(parentComponent);
+    
     return dialog;
   }
 
@@ -513,6 +509,8 @@ public class JOptionPane extends JComponent implements Accessible
    */
   public Object getInputValue()
   {
+    if (getValue().equals(new Integer(CANCEL_OPTION)))
+      setInputValue(null);
     return inputValue;
   }
 
@@ -858,13 +856,13 @@ public class JOptionPane extends JComponent implements Accessible
    */
   public static int showConfirmDialog(Component parentComponent, Object message)
   {
-    JOptionPane pane = new JOptionPane(message);
+    JOptionPane pane = new JOptionPane(message, QUESTION_MESSAGE);
     JDialog dialog = pane.createDialog(parentComponent, "Select an Option");
-
-    dialog.pack();
     dialog.show();
-
-    return ((Integer) pane.getValue()).intValue();
+    
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -886,10 +884,11 @@ public class JOptionPane extends JComponent implements Accessible
   {
     JOptionPane pane = new JOptionPane(message, PLAIN_MESSAGE, optionType);
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
 
-    return ((Integer) pane.getValue()).intValue();
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -912,10 +911,11 @@ public class JOptionPane extends JComponent implements Accessible
   {
     JOptionPane pane = new JOptionPane(message, messageType, optionType);
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
 
-    return ((Integer) pane.getValue()).intValue();
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -940,10 +940,11 @@ public class JOptionPane extends JComponent implements Accessible
   {
     JOptionPane pane = new JOptionPane(message, messageType, optionType, icon);
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
 
-    return ((Integer) pane.getValue()).intValue();
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -964,9 +965,8 @@ public class JOptionPane extends JComponent implements Accessible
     JOptionPane pane = new JOptionPane(message, QUESTION_MESSAGE);
     pane.setWantsInput(true);
     JDialog dialog = pane.createDialog(parentComponent, null);
-    dialog.pack();
     dialog.show();
-
+    
     return (String) pane.getInputValue();
   }
 
@@ -991,9 +991,8 @@ public class JOptionPane extends JComponent implements Accessible
     pane.setInitialSelectionValue(initialSelectionValue);
     pane.setWantsInput(true);
     JDialog dialog = pane.createDialog(parentComponent, null);
-    dialog.pack();
     dialog.show();
-
+    
     return (String) pane.getInputValue();
   }
 
@@ -1017,9 +1016,8 @@ public class JOptionPane extends JComponent implements Accessible
     JOptionPane pane = new JOptionPane(message, messageType);
     pane.setWantsInput(true);
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
-
+    
     return (String) pane.getInputValue();
   }
 
@@ -1050,10 +1048,9 @@ public class JOptionPane extends JComponent implements Accessible
     pane.setSelectionValues(selectionValues);
     pane.setInitialSelectionValue(initialSelectionValue);
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
-
-    return (String) pane.getInputValue();
+    
+    return pane.getInputValue();
   }
 
   /**
@@ -1071,9 +1068,8 @@ public class JOptionPane extends JComponent implements Accessible
     JOptionPane pane = new JOptionPane(message, QUESTION_MESSAGE);
     pane.setWantsInput(true);
     JDialog dialog = pane.createDialog(null, null);
-    dialog.pack();
     dialog.show();
-
+    
     return (String) pane.getInputValue();
   }
 
@@ -1096,9 +1092,8 @@ public class JOptionPane extends JComponent implements Accessible
     pane.setWantsInput(true);
     pane.setInitialSelectionValue(initialSelectionValue);
     JDialog dialog = pane.createDialog(null, null);
-    dialog.pack();
     dialog.show();
-
+    
     return (String) pane.getInputValue();
   }
 
@@ -1120,8 +1115,10 @@ public class JOptionPane extends JComponent implements Accessible
     JInternalFrame frame = pane.createInternalFrame(parentComponent, null);
 
     startModal(frame);
-
-    return ((Integer) pane.getValue()).intValue();
+    
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -1146,7 +1143,9 @@ public class JOptionPane extends JComponent implements Accessible
 
     startModal(frame);
 
-    return ((Integer) pane.getValue()).intValue();
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -1172,7 +1171,9 @@ public class JOptionPane extends JComponent implements Accessible
 
     startModal(frame);
 
-    return ((Integer) pane.getValue()).intValue();
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -1200,7 +1201,9 @@ public class JOptionPane extends JComponent implements Accessible
 
     startModal(frame);
 
-    return ((Integer) pane.getValue()).intValue();
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -1222,7 +1225,7 @@ public class JOptionPane extends JComponent implements Accessible
     JInternalFrame frame = pane.createInternalFrame(parentComponent, null);
 
     startModal(frame);
-
+    
     return (String) pane.getInputValue();
   }
 
@@ -1248,7 +1251,7 @@ public class JOptionPane extends JComponent implements Accessible
     JInternalFrame frame = pane.createInternalFrame(parentComponent, title);
 
     startModal(frame);
-
+    
     return (String) pane.getInputValue();
   }
 
@@ -1283,8 +1286,8 @@ public class JOptionPane extends JComponent implements Accessible
     JInternalFrame frame = pane.createInternalFrame(parentComponent, title);
 
     startModal(frame);
-
-    return (String) pane.getInputValue();
+    
+    return pane.getInputValue();
   }
 
   /**
@@ -1376,8 +1379,10 @@ public class JOptionPane extends JComponent implements Accessible
     JInternalFrame frame = pane.createInternalFrame(parentComponent, title);
 
     startModal(frame);
-
-    return ((Integer) pane.getValue()).intValue();
+ 
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -1391,8 +1396,7 @@ public class JOptionPane extends JComponent implements Accessible
   {
     JOptionPane pane = new JOptionPane(message, INFORMATION_MESSAGE);
     JDialog dialog = pane.createDialog(parentComponent, null);
-    dialog.pack();
-    dialog.show();
+    dialog.show();   
   }
 
   /**
@@ -1410,7 +1414,6 @@ public class JOptionPane extends JComponent implements Accessible
   {
     JOptionPane pane = new JOptionPane(message, messageType);
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
   }
 
@@ -1431,7 +1434,6 @@ public class JOptionPane extends JComponent implements Accessible
     JOptionPane pane = new JOptionPane(message, messageType);
     pane.setIcon(icon);
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
   }
 
@@ -1461,10 +1463,11 @@ public class JOptionPane extends JComponent implements Accessible
                                        options, initialValue);
 
     JDialog dialog = pane.createDialog(parentComponent, title);
-    dialog.pack();
     dialog.show();
 
-    return ((Integer) pane.getValue()).intValue();
+    if (pane.getValue() instanceof Integer)
+      return ((Integer) pane.getValue()).intValue();
+    return -1;
   }
 
   /**
@@ -1473,7 +1476,6 @@ public class JOptionPane extends JComponent implements Accessible
   public void updateUI()
   {
     setUI((OptionPaneUI) UIManager.getUI(this));
-    invalidate();
   }
 
   /**
@@ -1524,34 +1526,34 @@ public class JOptionPane extends JComponent implements Accessible
    * JInternalFrame's preferred size.
    *
    * @param f The JInternalFrame to make modal.
-   * @param pane The JOptionPane to add to the JInternalFrame.
    */
   private static void startModal(JInternalFrame f)
   {
     synchronized (f)
-      {
-	final JInternalFrame tmp = f;
-	tmp.toFront();
+    {
+      final JInternalFrame tmp = f;
+      tmp.toFront();
 
-	f.addInternalFrameListener(new InternalFrameAdapter()
-	    {
-	      public void internalFrameClosed(InternalFrameEvent e)
-	      {
-		synchronized (tmp)
-		  {
-		    tmp.removeInternalFrameListener(this);
-		    tmp.notifyAll();
-		  }
-	      }
-	    });
-	try
-	  {
-	    while (! f.isClosed())
-	      f.wait();
-	  }
-	catch (InterruptedException ignored)
-	  {
-	  }
-      }
+      f.addInternalFrameListener(new InternalFrameAdapter()
+                                 {
+                                   public void internalFrameClosed(InternalFrameEvent e)
+                                   {
+                                     synchronized (tmp)
+                                     {
+                                       tmp.removeInternalFrameListener(this);
+                                       tmp.notifyAll();
+                                     }
+                                   }
+                                 });
+      try
+        {
+          while (! f.isClosed())
+            f.wait();
+        }
+      catch (InterruptedException ignored)
+        {
+          // Ignore this Exception.
+        }
+    }
   }
 }
