@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -52,12 +52,12 @@ import javax.swing.AbstractAction;
 import javax.swing.AbstractButton;
 import javax.swing.ButtonModel;
 import javax.swing.JComponent;
+import javax.swing.SwingUtilities;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 
-public class BasicButtonListener
-  implements MouseListener, MouseMotionListener, FocusListener, 
-             ChangeListener, PropertyChangeListener
+public class BasicButtonListener implements MouseListener, MouseMotionListener,
+  FocusListener, ChangeListener, PropertyChangeListener
 {
   public BasicButtonListener(AbstractButton b)
   {
@@ -66,10 +66,12 @@ public class BasicButtonListener
   
   public void propertyChange(PropertyChangeEvent e)
   {
+    // TODO: What should be done here, if anything?
   }
   
   protected void checkOpacity(AbstractButton b) 
   {    
+    // TODO: What should be done here?
   }
   
   public void focusGained(FocusEvent e) 
@@ -129,18 +131,22 @@ public class BasicButtonListener
   
   public void stateChanged(ChangeEvent e)
   {
+    // TODO: What should be done here, if anything?
   }
   
   public void mouseMoved(MouseEvent e)
   {
+    // TODO: What should be done here, if anything?
   }
   
   public void mouseDragged(MouseEvent e)
   {
+    // TODO: What should be done here, if anything?
   }
   
   public void mouseClicked(MouseEvent e)
   {
+    // TODO: What should be done here, if anything?
   }
 
   /**
@@ -199,14 +205,12 @@ public class BasicButtonListener
       {
         AbstractButton button = (AbstractButton) e.getSource();
         ButtonModel model = button.getModel();
-        if (button.isRolloverEnabled())
+        if (button.isRolloverEnabled()
+            && ! SwingUtilities.isLeftMouseButton(e))
           model.setRollover(true);
-        
-        if (model.isPressed() 
-            && (e.getModifiersEx() & InputEvent.BUTTON1_DOWN_MASK) != 0)
+
+        if (model.isPressed())
           model.setArmed(true);
-        else
-          model.setArmed(false);
       }
   }
 
