@@ -66,7 +66,6 @@ import javax.swing.JMenuBar;
 import javax.swing.JMenuItem;
 import javax.swing.SwingConstants;
 import javax.swing.SwingUtilities;
-import javax.swing.UIDefaults;
 import javax.swing.UIManager;
 
 /**
@@ -83,6 +82,14 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   public class CloseAction extends AbstractAction
   {
+    /**
+     * Creates a new action.
+     */
+    public CloseAction()
+    {
+      super("Close");
+    }
+    
     /**
      * This method is called when something closes the JInternalFrame.
      *
@@ -92,13 +99,14 @@ public class BasicInternalFrameTitlePane extends JComponent
     {
       if (frame.isClosable())
         {
-	  try
-	    {
-	      frame.setClosed(true);
-	    }
-	  catch (PropertyVetoException pve)
-	    {
-	    }
+          try
+            {
+              frame.setClosed(true);
+            }
+          catch (PropertyVetoException pve)
+            {
+              // We do nothing if the attempt has been vetoed.
+            }
         }
     }
   }
@@ -112,6 +120,14 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   public class IconifyAction extends AbstractAction
   {
+    /**
+     * Creates a new action.
+     */
+    public IconifyAction()
+    {
+      super("Minimize");
+    }
+
     /**
      * This method is called when the user wants to iconify the
      * JInternalFrame.
@@ -122,13 +138,14 @@ public class BasicInternalFrameTitlePane extends JComponent
     {
       if (frame.isIconifiable() && ! frame.isIcon())
         {
-	  try
-	    {
-	      frame.setIcon(true);
-	    }
-	  catch (PropertyVetoException pve)
-	    {
-	    }
+          try
+            {
+              frame.setIcon(true);
+            }
+          catch (PropertyVetoException pve)
+            {
+              // We do nothing if the attempt has been vetoed.
+            }
         }
     }
   }
@@ -142,6 +159,13 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   public class MaximizeAction extends AbstractAction
   {
+    /**
+     * Creates a new action.
+     */
+    public MaximizeAction()
+    {
+      super("Maximize");
+    }
     /**
      * This method is called when the user wants to maximize the
      * JInternalFrame.
@@ -152,13 +176,14 @@ public class BasicInternalFrameTitlePane extends JComponent
     {
       try
         {
-	  if (frame.isMaximizable() && ! frame.isMaximum())
-	    frame.setMaximum(true);
-	  else if (frame.isMaximum())
-	    frame.setMaximum(false);
+          if (frame.isMaximizable() && ! frame.isMaximum())
+            frame.setMaximum(true);
+          else if (frame.isMaximum())
+            frame.setMaximum(false);
         }
       catch (PropertyVetoException pve)
         {
+          // We do nothing if the attempt has been vetoed.
         }
     }
   }
@@ -172,6 +197,13 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   public class MoveAction extends AbstractAction
   {
+    /**
+     * Creates a new action.
+     */
+    public MoveAction()
+    {
+      super("Move");
+    }
     /**
      * This method is called when the user wants to drag the JInternalFrame.
      *
@@ -193,6 +225,13 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   public class RestoreAction extends AbstractAction
   {
+    /**
+     * Creates a new action.
+     */
+    public RestoreAction()
+    {
+      super("Restore");
+    }
     /**
      * This method is called when the user wants to restore the
      * JInternalFrame.
@@ -203,13 +242,14 @@ public class BasicInternalFrameTitlePane extends JComponent
     {
       if (frame.isMaximum())
         {
-	  try
-	    {
-	      frame.setMaximum(false);
-	    }
-	  catch (PropertyVetoException pve)
-	    {
-	    }
+          try
+            {
+              frame.setMaximum(false);
+            }
+          catch (PropertyVetoException pve)
+            {
+              // We do nothing if the attempt has been vetoed.
+            }
         }
     }
   }
@@ -223,6 +263,13 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   public class SizeAction extends AbstractAction
   {
+    /**
+     * Creates a new action.
+     */
+    public SizeAction()
+    {
+      super("Size");
+    }
     /**
      * This method is called when the user wants to resize the JInternalFrame.
      *
@@ -254,27 +301,27 @@ public class BasicInternalFrameTitlePane extends JComponent
     {
       String propName = evt.getPropertyName();
       if (propName.equals("closable"))
-	{
-	  if (evt.getNewValue().equals(Boolean.TRUE))
-	    closeButton.setVisible(true);
-	  else
-	    closeButton.setVisible(false);
-	}
-      else if (propName.equals("iconifiable"))
-	{
-	  if (evt.getNewValue().equals(Boolean.TRUE))
-	    iconButton.setVisible(true);
-	  else
-	    iconButton.setVisible(false);
-	}
+        {
+          if (evt.getNewValue().equals(Boolean.TRUE))
+            closeButton.setVisible(true);
+          else
+            closeButton.setVisible(false);
+        }
+      else if (propName.equals("iconable"))
+        {
+          if (evt.getNewValue().equals(Boolean.TRUE))
+            iconButton.setVisible(true);
+          else
+            iconButton.setVisible(false);
+        }
       else if (propName.equals("maximizable"))
-	{
-	  if (evt.getNewValue().equals(Boolean.TRUE))
-	    maxButton.setVisible(true);
-	  else
-	    maxButton.setVisible(false);
-	}
-	
+        {
+          if (evt.getNewValue().equals(Boolean.TRUE))
+            maxButton.setVisible(true);
+          else
+            maxButton.setVisible(false);
+        }
+      enableActions();
     }
   }
 
@@ -293,7 +340,7 @@ public class BasicInternalFrameTitlePane extends JComponent
      *
      * @return True if this Component can receive focus.
      */
-    public boolean isFocusTransversable()
+    public boolean isFocusTraversable()
     {
       return true;
     }
@@ -377,24 +424,26 @@ public class BasicInternalFrameTitlePane extends JComponent
 
       int loc = width + insets.left - 1;
       int top = insets.top + 1;
-      int buttonWidth = height - 2;
       int buttonHeight = height - 4;
       if (closeButton.isVisible())
         {
-	  loc -= buttonWidth + 2;
-	  closeButton.setBounds(loc, top, buttonWidth, buttonHeight);
+          int buttonWidth = closeIcon.getIconWidth();
+          loc -= buttonWidth + 2;
+          closeButton.setBounds(loc, top, buttonWidth, buttonHeight);
         }
 
       if (maxButton.isVisible())
         {
-	  loc -= buttonWidth + 2;
-	  maxButton.setBounds(loc, top, buttonWidth, buttonHeight);
+          int buttonWidth = maxIcon.getIconWidth();
+          loc -= buttonWidth + 2;
+          maxButton.setBounds(loc, top, buttonWidth, buttonHeight);
         }
 
       if (iconButton.isVisible())
         {
-	  loc -= buttonWidth + 2;
-	  iconButton.setBounds(loc, top, buttonWidth, buttonHeight);
+          int buttonWidth = iconIcon.getIconWidth();
+          loc -= buttonWidth + 2;
+          iconButton.setBounds(loc, top, buttonWidth, buttonHeight);
         }
 
       if (title != null)
@@ -435,6 +484,7 @@ public class BasicInternalFrameTitlePane extends JComponent
      */
     public void removeLayoutComponent(Component c)
     {
+      // Nothing to do here.
     }
   }
 
@@ -466,25 +516,26 @@ public class BasicInternalFrameTitlePane extends JComponent
       // These buttons cannot be given focus.
       return false;
     }
+
   }
 
   /** The action command for the Close action. */
-  protected static final String CLOSE_CMD = "Close";
+  protected static final String CLOSE_CMD;
 
   /** The action command for the Minimize action. */
-  protected static final String ICONIFY_CMD = "Minimize";
+  protected static final String ICONIFY_CMD;
 
   /** The action command for the Maximize action. */
-  protected static final String MAXIMIZE_CMD = "Maximize";
+  protected static final String MAXIMIZE_CMD;
 
   /** The action command for the Move action. */
-  protected static final String MOVE_CMD = "Move";
+  protected static final String MOVE_CMD;
 
   /** The action command for the Restore action. */
-  protected static final String RESTORE_CMD = "Restore";
+  protected static final String RESTORE_CMD;
 
   /** The action command for the Size action. */
-  protected static final String SIZE_CMD = "Size";
+  protected static final String SIZE_CMD;
 
   /** The action associated with closing the JInternalFrame. */
   protected Action closeAction;
@@ -522,6 +573,9 @@ public class BasicInternalFrameTitlePane extends JComponent
   /** The icon displayed in the iconify button. */
   protected Icon iconIcon = BasicIconFactory.createEmptyFrameIcon();
 
+  /** The icon displayed in the close button. */
+  protected Icon closeIcon;
+  
   /** The JInternalFrame that this TitlePane is used in. */
   protected JInternalFrame frame;
 
@@ -560,6 +614,17 @@ public class BasicInternalFrameTitlePane extends JComponent
    * This is package-private to avoid an accessor method.
    */
   transient JLabel title;
+  
+  static
+    {
+      // not constants in JDK
+      CLOSE_CMD = "Close";
+      ICONIFY_CMD = "Minimize";
+      MAXIMIZE_CMD = "Maximize";
+      MOVE_CMD = "Move";
+      RESTORE_CMD = "Restore";
+      SIZE_CMD = "Size";
+    }
 
   /**
    * Creates a new BasicInternalFrameTitlePane object that is used in the
@@ -645,7 +710,7 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   protected void installListeners()
   {
-    propertyChangeListener = new PropertyChangeHandler();
+    propertyChangeListener = createPropertyChangeListener();
     frame.addPropertyChangeListener(propertyChangeListener);
   }
 
@@ -663,14 +728,15 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   protected void installDefaults()
   {
-    // FIXME: move icons to defaults.
-    UIDefaults defaults = UIManager.getLookAndFeelDefaults();
-
-    setFont(defaults.getFont("InternalFrame.titleFont"));
-    selectedTextColor = defaults.getColor("InternalFrame.activeTitleForeground");
-    selectedTitleColor = defaults.getColor("InternalFrame.activeTitleBackground");
-    notSelectedTextColor = defaults.getColor("InternalFrame.inactiveTitleForeground");
-    notSelectedTitleColor = defaults.getColor("InternalFrame.inactiveTitleBackground");
+    title.setFont(UIManager.getFont("InternalFrame.titleFont"));
+    selectedTextColor = UIManager.getColor("InternalFrame.activeTitleForeground");
+    selectedTitleColor = UIManager.getColor("InternalFrame.activeTitleBackground");
+    notSelectedTextColor = UIManager.getColor("InternalFrame.inactiveTitleForeground");
+    notSelectedTitleColor = UIManager.getColor("InternalFrame.inactiveTitleBackground");
+  
+    closeIcon = UIManager.getIcon("InternalFrame.closeIcon");
+    iconIcon = UIManager.getIcon("InternalFrame.iconifyIcon");
+    maxIcon = UIManager.getIcon("InternalFrame.maximizeIcon");
   }
 
   /**
@@ -683,6 +749,10 @@ public class BasicInternalFrameTitlePane extends JComponent
     selectedTitleColor = null;
     notSelectedTextColor = null;
     notSelectedTitleColor = null;
+    
+    closeIcon = null;
+    iconIcon = null;
+    maxIcon = null;
   }
 
   /**
@@ -691,12 +761,15 @@ public class BasicInternalFrameTitlePane extends JComponent
   protected void createButtons()
   {
     closeButton = new PaneButton(closeAction);
+    closeButton.setText(null);
     if (!frame.isClosable())
       closeButton.setVisible(false);
     iconButton = new PaneButton(iconifyAction);
+    iconButton.setText(null);
     if (!frame.isIconifiable())
       iconButton.setVisible(false);
     maxButton = new PaneButton(maximizeAction);
+    maxButton.setText(null);
     if (!frame.isMaximizable())
       maxButton.setVisible(false);
   }
@@ -706,15 +779,12 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   protected void setButtonIcons()
   {
-    Icon icon = UIManager.getIcon("InternalFrame.closeIcon");
-    if (icon != null)
-      closeButton.setIcon(icon);
-    icon = UIManager.getIcon("InternalFrame.iconifyIcon");
-    if (icon != null)
-      iconButton.setIcon(icon);
-    icon = UIManager.getIcon("InternalFrame.maximizeIcon");
-    if (icon != null)
-      maxButton.setIcon(icon);
+    if (closeIcon != null && closeButton != null)
+      closeButton.setIcon(closeIcon);
+    if (iconIcon != null && iconButton != null)
+      iconButton.setIcon(iconIcon);
+    if (maxIcon != null && maxButton != null)
+      maxButton.setIcon(maxIcon);
   }
 
   /**
@@ -816,11 +886,12 @@ public class BasicInternalFrameTitlePane extends JComponent
   public void paintComponent(Graphics g)
   {
     paintTitleBackground(g);
-    Font f = g.getFont();
-    FontMetrics fm = g.getFontMetrics(f);
     if (frame.getTitle() != null && title != null)
       {
 	Color saved = g.getColor();
+        Font f = title.getFont();
+        g.setFont(f);
+        FontMetrics fm = g.getFontMetrics(f);
 	if (frame.isSelected())
 	  g.setColor(selectedTextColor);
 	else
@@ -838,6 +909,9 @@ public class BasicInternalFrameTitlePane extends JComponent
    */
   protected void paintTitleBackground(Graphics g)
   {
+    if (!isOpaque())
+      return;
+
     Color saved = g.getColor();
     Dimension dims = getSize();
 
