@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package javax.swing.plaf.basic;
 
+import gnu.classpath.NotImplementedException;
+
 import java.awt.Canvas;
 import java.awt.Color;
 import java.awt.Component;
@@ -58,10 +60,11 @@ import java.beans.PropertyChangeListener;
 import javax.swing.JComponent;
 import javax.swing.JSplitPane;
 import javax.swing.KeyStroke;
-import javax.swing.UIDefaults;
+import javax.swing.LookAndFeel;
 import javax.swing.UIManager;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.SplitPaneUI;
+import javax.swing.plaf.UIResource;
 
 /**
  * This is the Basic Look and Feel implementation of the SplitPaneUI  class.
@@ -99,6 +102,16 @@ public class BasicSplitPaneUI extends SplitPaneUI
      */
     protected int[] sizes = new int[3];
 
+    /**
+     * Creates a new instance. This is package private because the reference
+     * implementation has no public constructor either. Still, we need to
+     * call it from BasicVerticalLayoutManager.
+     */
+    BasicHorizontalLayoutManager()
+    {
+      // Nothing to do here.
+    }
+
     /**
      * This method adds the component given to the JSplitPane. The position of
      * the component is given by the constraints object.
@@ -126,14 +139,14 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       int i = 0;
       if (place == null)
-	i = 2;
+        i = 2;
       else if (place.equals(JSplitPane.TOP) || place.equals(JSplitPane.LEFT))
-	i = 0;
+        i = 0;
       else if (place.equals(JSplitPane.BOTTOM)
                || place.equals(JSplitPane.RIGHT))
-	i = 1;
+        i = 1;
       else
-	throw new IllegalArgumentException("Illegal placement in JSplitPane");
+        throw new IllegalArgumentException("Illegal placement in JSplitPane");
       components[i] = component;
       resetSizeAt(i);
       splitPane.revalidate();
@@ -164,7 +177,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
     protected int getInitialLocation(Insets insets)
     {
       if (insets != null)
-	return insets.left;
+        return insets.left;
       return 0;
     }
 
@@ -205,7 +218,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       Dimension dims = c.getPreferredSize();
       if (dims != null)
-	return dims.width;
+        return dims.width;
       return 0;
     }
 
@@ -250,23 +263,24 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       if (container instanceof JSplitPane)
         {
-	  JSplitPane split = (JSplitPane) container;
-	  distributeExtraSpace();
-	  Insets insets = split.getInsets();
-	  int width = getInitialLocation(insets);
-	  Dimension dims = split.getSize();
-	  for (int i = 0; i < components.length; i += 2)
-	    {
-	      if (components[i] == null)
-		continue;
-	      setComponentToSize(components[i], sizes[i], width, insets, dims);
-	      width += sizes[i];
-	    }
-	  if (components[1] != null)
-	    {
-	      setComponentToSize(components[1], sizes[1], width, insets, dims);
-	      width += sizes[1];
-	    }
+          JSplitPane split = (JSplitPane) container;
+          distributeExtraSpace();
+          Insets insets = split.getInsets();
+          Dimension dims = split.getSize();
+          int loc = getInitialLocation(insets);
+          int available = getAvailableSize(dims, insets);
+          sizes[0] = getDividerLocation(split) - loc;
+          sizes[1] = available - sizes[0] - sizes[2];
+          // The size of the divider won't change.
+
+          // Layout component#1.
+          setComponentToSize(components[0], sizes[0], loc, insets, dims);
+          // Layout divider.
+          loc += sizes[0];
+          setComponentToSize(components[2], sizes[2], loc, insets, dims);
+          // Layout component#2. 
+          loc += sizes[2];
+          setComponentToSize(components[1], sizes[1], loc, insets, dims);
         }
     }
 
@@ -297,23 +311,23 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       if (target instanceof JSplitPane)
         {
-	  JSplitPane split = (JSplitPane) target;
-	  Insets insets = target.getInsets();
+          JSplitPane split = (JSplitPane) target;
+          Insets insets = target.getInsets();
 
-	  int height = 0;
-	  int width = 0;
-	  for (int i = 0; i < components.length; i++)
-	    {
-	      if (components[i] == null)
-		continue;
-	      Dimension dims = components[i].getMinimumSize();
-	      if (dims != null)
-	        {
-		  width += dims.width;
-		  height = Math.max(height, dims.height);
-	        }
-	    }
-	  return new Dimension(width, height);
+          int height = 0;
+          int width = 0;
+          for (int i = 0; i < components.length; i++)
+            {
+              if (components[i] == null)
+                continue;
+              Dimension dims = components[i].getMinimumSize();
+              if (dims != null)
+                {
+                  width += dims.width;
+                  height = Math.max(height, dims.height);
+                }
+            }
+          return new Dimension(width, height);
         }
       return null;
     }
@@ -331,24 +345,24 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       if (target instanceof JSplitPane)
         {
-	  JSplitPane split = (JSplitPane) target;
-	  Insets insets = target.getInsets();
+          JSplitPane split = (JSplitPane) target;
+          Insets insets = target.getInsets();
 
-	  int height = 0;
-	  int width = 0;
-	  for (int i = 0; i < components.length; i++)
-	    {
-	      if (components[i] == null)
-		continue;
-	      Dimension dims = components[i].getPreferredSize();
-	      if (dims != null)
-	        {
-		  width += dims.width;
-		  if (! (components[i] instanceof BasicSplitPaneDivider))
-		    height = Math.max(height, dims.height);
-	        }
-	    }
-	  return new Dimension(width, height);	
+          int height = 0;
+          int width = 0;
+          for (int i = 0; i < components.length; i++)
+            {
+              if (components[i] == null)
+                continue;
+              Dimension dims = components[i].getPreferredSize();
+              if (dims != null)
+                {
+                  width += dims.width;
+                  if (!(components[i] instanceof BasicSplitPaneDivider))
+                    height = Math.max(height, dims.height);
+                }
+            }
+          return new Dimension(width, height);
         }
       return null;
     }
@@ -362,11 +376,11 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       for (int i = 0; i < components.length; i++)
         {
-	  if (component == components[i])
-	    {
-	      components[i] = null;
-	      sizes[i] = 0;
-	    }
+          if (component == components[i])
+            {
+              components[i] = null;
+              sizes[i] = 0;
+            }
         }
     }
 
@@ -378,7 +392,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
     protected void resetSizeAt(int index)
     {
       if (components[index] != null)
-	sizes[index] = getPreferredSizeOfComponent(components[index]);
+        sizes[index] = getPreferredSizeOfComponent(components[index]);
     }
 
     /**
@@ -387,7 +401,9 @@ public class BasicSplitPaneUI extends SplitPaneUI
     public void resetToPreferredSizes()
     {
       for (int i = 0; i < components.length; i++)
-	resetSizeAt(i);
+        resetSizeAt(i);
+      setDividerLocation(splitPane,
+                         getInitialLocation(splitPane.getInsets()) + sizes[0]);
     }
 
     /**
@@ -404,7 +420,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
      */
     protected void setComponentToSize(Component c, int size, int location,
                                       Insets insets, Dimension containerSize)
-    {
+    { 
       int w = size;
       int h = containerSize.height - insets.top - insets.bottom;
       int x = location;
@@ -433,13 +449,13 @@ public class BasicSplitPaneUI extends SplitPaneUI
 
       if (left != null)
         {
-	  components[0] = left;
-	  resetSizeAt(0);
+          components[0] = left;
+          resetSizeAt(0);
         }
       if (right != null)
         {
-	  components[1] = right;
-	  resetSizeAt(1);
+          components[1] = right;
+          resetSizeAt(1);
         }
       components[2] = divider;
       resetSizeAt(2);
@@ -451,21 +467,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
      */
     void distributeExtraSpace()
     {
-      int availSize = getAvailableSize(splitPane.getSize(),
-                                       splitPane.getInsets());
-      int[] newSizes = new int[3];
-      double weight = splitPane.getResizeWeight();
-
-      int oldLen = sizes[0] + sizes[1];
-
-      // dividers don't change size.
-      availSize -= sizes[2] + oldLen;
-
-      int rightAlloc = (int) (availSize * (1 - weight));
-      int leftAlloc = availSize - rightAlloc;
-
-      sizes[0] += leftAlloc;
-      sizes[1] += rightAlloc;
+      // FIXME: This needs to be reimplemented correctly.
     }
 
     /**
@@ -480,9 +482,9 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       Dimension dims = components[index].getMinimumSize();
       if (dims != null)
-	return dims.width;
+        return dims.width;
       else
-	return 0;
+        return 0;
     }
   } //end BasicHorizontalLayoutManager
 
@@ -534,7 +536,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       Dimension dims = c.getPreferredSize();
       if (dims != null)
-	return dims.height;
+        return dims.height;
       return 0;
     }
 
@@ -563,23 +565,23 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       if (container instanceof JSplitPane)
         {
-	  JSplitPane split = (JSplitPane) container;
-	  Insets insets = container.getInsets();
+          JSplitPane split = (JSplitPane) container;
+          Insets insets = container.getInsets();
 
-	  int height = 0;
-	  int width = 0;
-	  for (int i = 0; i < components.length; i++)
-	    {
-	      if (components[i] == null)
-		continue;
-	      Dimension dims = components[i].getMinimumSize();
-	      if (dims != null)
-	        {
-		  height += dims.height;
-		  width = Math.max(width, dims.width);
-	        }
-	    }
-	  return new Dimension(width, height);
+          int height = 0;
+          int width = 0;
+          for (int i = 0; i < components.length; i++)
+            {
+              if (components[i] == null)
+                continue;
+              Dimension dims = components[i].getMinimumSize();
+              if (dims != null)
+                {
+                  height += dims.height;
+                  width = Math.max(width, dims.width);
+                }
+            }
+          return new Dimension(width, height);
         }
       return null;
     }
@@ -597,23 +599,23 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       if (container instanceof JSplitPane)
         {
-	  JSplitPane split = (JSplitPane) container;
-	  Insets insets = container.getInsets();
+          JSplitPane split = (JSplitPane) container;
+          Insets insets = container.getInsets();
 
-	  int height = 0;
-	  int width = 0;
-	  for (int i = 0; i < components.length; i++)
-	    {
-	      if (components[i] == null)
-		continue;
-	      Dimension dims = components[i].getPreferredSize();
-	      if (dims != null)
-	        {
-		  height += dims.height;
-		  width = Math.max(width, dims.width);
-	        }
-	    }
-	  return new Dimension(width, height);
+          int height = 0;
+          int width = 0;
+          for (int i = 0; i < components.length; i++)
+            {
+              if (components[i] == null)
+                continue;
+              Dimension dims = components[i].getPreferredSize();
+              if (dims != null)
+                {
+                  height += dims.height;
+                  width = Math.max(width, dims.width);
+                }
+            }
+          return new Dimension(width, height);
         }
       return null;
     }
@@ -637,7 +639,6 @@ public class BasicSplitPaneUI extends SplitPaneUI
       int x = insets.left;
       int h = size;
       int w = containerSize.width - insets.left - insets.right;
-
       c.setBounds(x, y, w, h);
     }
 
@@ -653,9 +654,9 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       Dimension dims = components[index].getMinimumSize();
       if (dims != null)
-	return dims.height;
+        return dims.height;
       else
-	return 0;
+        return 0;
     }
   }
 
@@ -814,46 +815,41 @@ public class BasicSplitPaneUI extends SplitPaneUI
     {
       if (e.getPropertyName().equals(JSplitPane.DIVIDER_SIZE_PROPERTY))
         {
-	  int newSize = splitPane.getDividerSize();
-	  int[] tmpSizes = layoutManager.getSizes();
-	  dividerSize = tmpSizes[2];
-	  Component left = splitPane.getLeftComponent();
-	  Component right = splitPane.getRightComponent();
-	  int newSpace = newSize - tmpSizes[2];
+          int newSize = splitPane.getDividerSize();
+          int[] tmpSizes = layoutManager.getSizes();
+          dividerSize = tmpSizes[2];
+          int newSpace = newSize - tmpSizes[2];
+          tmpSizes[2] = newSize;
 
-	  tmpSizes[2] = newSize;
-
-	  tmpSizes[0] += newSpace / 2;
-	  tmpSizes[1] += newSpace / 2;
-
-	  layoutManager.setSizes(tmpSizes);
+          tmpSizes[0] += newSpace / 2;
+          tmpSizes[1] += newSpace / 2;
+      
+          layoutManager.setSizes(tmpSizes);
         }
       else if (e.getPropertyName().equals(JSplitPane.ORIENTATION_PROPERTY))
         {
-	  int max = layoutManager.getAvailableSize(splitPane.getSize(),
-	                                           splitPane.getInsets());
-	  int dividerLoc = getDividerLocation(splitPane);
-	  double prop = ((double) dividerLoc) / max;
+          int max = layoutManager.getAvailableSize(splitPane.getSize(),
+                                                   splitPane.getInsets());
+          int dividerLoc = getDividerLocation(splitPane);
+          double prop = ((double) dividerLoc) / max;
 
-	  resetLayoutManager();
-	  if (prop <= 1 && prop >= 0)
-	    splitPane.setDividerLocation(prop);
+          resetLayoutManager();
+          if (prop <= 1 && prop >= 0)
+            splitPane.setDividerLocation(prop);
         }
-      layoutManager.layoutContainer(splitPane);
-      splitPane.repaint();
       // Don't have to deal with continuous_layout - only
       // necessary in dragging modes (and it's checked
       // every time you drag there)
       // Don't have to deal with resize_weight (as there
       // will be no extra space associated with this
       // event - the changes to the weighting will
-      // be taken into account the next time the 
+      // be taken into account the next time the
       // sizes change.)
-      // Don't have to deal with divider_location 
+      // Don't have to deal with divider_location
       // The method in JSplitPane calls our setDividerLocation
       // so we'll know about those anyway.
       // Don't have to deal with last_divider_location
-      // Although I'm not sure why, it doesn't seem to 
+      // Although I'm not sure why, it doesn't seem to
       // have any effect on Sun's JSplitPane.
       // one_touch_expandable changes are dealt with
       // by our divider.
@@ -937,11 +933,14 @@ public class BasicSplitPaneUI extends SplitPaneUI
   /** The JSplitPane that this UI draws. */
   protected JSplitPane splitPane;
 
+  private int dividerLocation;
+
   /**
    * Creates a new BasicSplitPaneUI object.
    */
   public BasicSplitPaneUI()
   {
+    // Nothing to do here.
   }
 
   /**
@@ -965,10 +964,10 @@ public class BasicSplitPaneUI extends SplitPaneUI
   {
     if (c instanceof JSplitPane)
       {
-	splitPane = (JSplitPane) c;
-	installDefaults();
-	installListeners();
-	installKeyboardActions();
+        splitPane = (JSplitPane) c;
+        installDefaults();
+        installListeners();
+        installKeyboardActions();
       }
   }
 
@@ -991,16 +990,17 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   protected void installDefaults()
   {
+    LookAndFeel.installColors(splitPane, "SplitPane.background",
+                              "SplitPane.foreground");
+    LookAndFeel.installBorder(splitPane, "SplitPane.border");
     divider = createDefaultDivider();
+    divider.setBorder(UIManager.getBorder("SplitPaneDivider.border"));
     resetLayoutManager();
     nonContinuousLayoutDivider = createDefaultNonContinuousLayoutDivider();
     splitPane.add(divider, JSplitPane.DIVIDER);
 
     // There is no need to add the nonContinuousLayoutDivider
-    UIDefaults defaults = UIManager.getLookAndFeelDefaults();
-    splitPane.setBackground(defaults.getColor("SplitPane.background"));
-    splitPane.setBorder(defaults.getBorder("SplitPane.border"));
-    splitPane.setDividerSize(defaults.getInt("SplitPane.dividerSize"));
+    splitPane.setDividerSize(UIManager.getInt("SplitPane.dividerSize"));
     splitPane.setOpaque(true);
   }
 
@@ -1015,8 +1015,10 @@ public class BasicSplitPaneUI extends SplitPaneUI
     divider = null;
     nonContinuousLayoutDivider = null;
 
-    splitPane.setBackground(null);
-    splitPane.setBorder(null);
+    if (splitPane.getBackground() instanceof UIResource)
+      splitPane.setBackground(null);
+    if (splitPane.getBorder() instanceof UIResource)
+      splitPane.setBorder(null);
   }
 
   /**
@@ -1047,6 +1049,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
    * This method installs the keyboard actions for the JSplitPane.
    */
   protected void installKeyboardActions()
+    throws NotImplementedException
   {
     // FIXME: implement.
   }
@@ -1055,6 +1058,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
    * This method reverses the work done in installKeyboardActions.
    */
   protected void uninstallKeyboardActions()
+    throws NotImplementedException
   {
     // FIXME: implement.
   }
@@ -1221,8 +1225,9 @@ public class BasicSplitPaneUI extends SplitPaneUI
   {
     if (nonContinuousLayoutDivider == null)
       {
-	nonContinuousLayoutDivider = new Canvas();
-	nonContinuousLayoutDivider.setBackground(Color.DARK_GRAY);
+        nonContinuousLayoutDivider = new Canvas();
+        Color c = UIManager.getColor("SplitPaneDivider.draggingColor");
+        nonContinuousLayoutDivider.setBackground(c);
       }
     return nonContinuousLayoutDivider;
   }
@@ -1301,16 +1306,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   public void setDividerLocation(JSplitPane jc, int location)
   {
-    setLastDragLocation(getDividerLocation(splitPane));
-    splitPane.setLastDividerLocation(getDividerLocation(splitPane));
-    int[] tmpSizes = layoutManager.getSizes();
-    tmpSizes[0] = location
-                  - layoutManager.getInitialLocation(splitPane.getInsets());
-    tmpSizes[1] = layoutManager.getAvailableSize(splitPane.getSize(),
-                                                 splitPane.getInsets())
-                  - tmpSizes[0] - tmpSizes[1];
-
-    layoutManager.setSizes(tmpSizes);
+    dividerLocation = location;
     splitPane.revalidate();
     splitPane.repaint();
   }
@@ -1324,8 +1320,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   public int getDividerLocation(JSplitPane jc)
   {
-    return layoutManager.sizes[0]
-           + layoutManager.getInitialLocation(splitPane.getInsets());
+    return dividerLocation;
   }
 
   /**
@@ -1338,11 +1333,9 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   public int getMinimumDividerLocation(JSplitPane jc)
   {
-    int value = layoutManager.getInitialLocation(jc.getInsets())
-                - layoutManager.getAvailableSize(jc.getSize(), jc.getInsets())
-                + splitPane.getDividerSize();
-    if (layoutManager.components[1] != null)
-      value += layoutManager.minimumSizeOfComponent(1);
+    int value = layoutManager.getInitialLocation(jc.getInsets());
+    if (layoutManager.components[0] != null)
+      value += layoutManager.minimumSizeOfComponent(0);
     return value;
   }
 
@@ -1388,6 +1381,7 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   public void paint(Graphics g, JComponent jc)
   {
+    // TODO: What should be done here?
   }
 
   /**
@@ -1461,22 +1455,22 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   protected void startDragging()
   {
+    Component left = splitPane.getLeftComponent();
+    Component right = splitPane.getRightComponent();
     dividerSize = divider.getDividerSize();
     setLastDragLocation(-1);
 
-    if (! splitPane.getLeftComponent().isLightweight()
-        || ! splitPane.getRightComponent().isLightweight())
+    if ((left != null && !left.isLightweight())
+        || (right != null && !right.isLightweight()))
       draggingHW = true;
 
     if (splitPane.isContinuousLayout())
       nonContinuousLayoutDivider.setVisible(false);
     else
       {
-	nonContinuousLayoutDivider.setVisible(true);
-	nonContinuousLayoutDivider.setBounds(divider.getBounds());
+        nonContinuousLayoutDivider.setVisible(true);
+        nonContinuousLayoutDivider.setBounds(divider.getBounds());
       }
-    splitPane.revalidate();
-    splitPane.repaint();
   }
 
   /**
@@ -1496,12 +1490,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
       splitPane.setDividerLocation(location);
     else
       {
-	Point p = nonContinuousLayoutDivider.getLocation();
-	if (getOrientation() == JSplitPane.HORIZONTAL_SPLIT)
-	  p.x = location;
-	else
-	  p.y = location;
-	nonContinuousLayoutDivider.setLocation(p);
+        Point p = nonContinuousLayoutDivider.getLocation();
+        if (getOrientation() == JSplitPane.HORIZONTAL_SPLIT)
+          p.x = location;
+        else
+          p.y = location;
+        nonContinuousLayoutDivider.setLocation(p);
       }
     setLastDragLocation(location);
     splitPane.repaint();
@@ -1518,11 +1512,9 @@ public class BasicSplitPaneUI extends SplitPaneUI
       nonContinuousLayoutDivider.setVisible(false);
     draggingHW = false;
     location = validLocation(location);
-    dragDividerTo(location);
     splitPane.setDividerLocation(location);
     splitPane.setLastDividerLocation(beginDragDividerLocation);
     beginDragDividerLocation = -1;
-    splitPane.repaint();
   }
 
   /**
@@ -1550,10 +1542,12 @@ public class BasicSplitPaneUI extends SplitPaneUI
    */
   private int validLocation(int location)
   {
-    if (location < getMinimumDividerLocation(splitPane))
-      return getMinimumDividerLocation(splitPane);
-    if (location > getMaximumDividerLocation(splitPane))
-      return getMaximumDividerLocation(splitPane);
+    int min = getMinimumDividerLocation(splitPane);
+    int max = getMaximumDividerLocation(splitPane);
+    if (min > 0 && location < min)
+      return min;
+    if (max > 0 && location > max)
+      return max;
     return location;
   }
 }
