@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package javax.swing.plaf.basic;
 
+import gnu.classpath.NotImplementedException;
+
 import java.awt.BorderLayout;
 import java.awt.Color;
 import java.awt.Component;
@@ -60,19 +62,21 @@ import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.util.Hashtable;
 
+import javax.swing.AbstractButton;
 import javax.swing.JButton;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
 import javax.swing.JFrame;
 import javax.swing.JToolBar;
+import javax.swing.KeyStroke;
+import javax.swing.LookAndFeel;
 import javax.swing.RootPaneContainer;
 import javax.swing.SwingConstants;
 import javax.swing.SwingUtilities;
-import javax.swing.UIDefaults;
 import javax.swing.UIManager;
 import javax.swing.border.Border;
+import javax.swing.border.EtchedBorder;
 import javax.swing.event.MouseInputListener;
-import javax.swing.plaf.BorderUIResource.EtchedBorderUIResource;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.ToolBarUI;
 import javax.swing.plaf.UIResource;
@@ -132,6 +136,26 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
   /** The Focus listener for the JToolBar. */
   protected FocusListener toolBarFocusListener;
 
+  /**
+   * @deprecated since JDK1.3.
+   */
+  protected KeyStroke leftKey;
+
+  /**
+   * @deprecated since JDK1.3.
+   */
+  protected KeyStroke rightKey;
+
+  /**
+   * @deprecated since JDK1.3.
+   */
+  protected KeyStroke upKey;
+
+  /**
+   * @deprecated since JDK1.3.
+   */
+  protected KeyStroke downKey;
+
   /**
    * The floating window that is responsible for holding the JToolBar when it
    * is dragged outside of its original parent.
@@ -286,7 +310,7 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
    */
   protected Border createNonRolloverBorder()
   {
-    return new EtchedBorderUIResource();
+    return new EtchedBorder();
   }
 
   /**
@@ -307,7 +331,7 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
    */
   protected Border createRolloverBorder()
   {
-    return new EtchedBorderUIResource()
+    return new EtchedBorder()
       {
 	public void paintBorder(Component c, Graphics g, int x, int y,
 	                        int width, int height)
@@ -383,6 +407,8 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
     boolean tmp = ((loc == SwingConstants.NORTH)
                   || (loc == SwingConstants.SOUTH) || (loc == -1));
 
+    cachedOrientation = toolBar.getOrientation();
+    cachedBounds = toolBar.getSize();
     if (((cachedOrientation == SwingConstants.HORIZONTAL) && tmp)
         || ((cachedOrientation == VERTICAL) && ! tmp))
       {
@@ -550,13 +576,11 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
 
     dragWindow = createDragWindow(toolBar);
 
-    cachedBounds = toolBar.getPreferredSize();
-    cachedOrientation = toolBar.getOrientation();
-
     nonRolloverBorder = createNonRolloverBorder();
     rolloverBorder = createRolloverBorder();
 
     borders = new Hashtable();
+    setRolloverBorders(toolBar.isRollover());
 
     fillHashtable();
   }
@@ -566,18 +590,15 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
    */
   protected void installDefaults()
   {
-    UIDefaults defaults = UIManager.getLookAndFeelDefaults();
+    LookAndFeel.installBorder(toolBar, "ToolBar.border");
+    LookAndFeel.installColorsAndFont(toolBar, "ToolBar.background",
+                                     "ToolBar.foreground", "ToolBar.font");
 
-    toolBar.setBorder(new ToolBarBorder());
-    toolBar.setBackground(defaults.getColor("ToolBar.background"));
-    toolBar.setForeground(defaults.getColor("ToolBar.foreground"));
-    toolBar.setFont(defaults.getFont("ToolBar.font"));
+    dockingBorderColor = UIManager.getColor("ToolBar.dockingForeground");
+    dockingColor = UIManager.getColor("ToolBar.dockingBackground");
 
-    dockingBorderColor = defaults.getColor("ToolBar.dockingForeground");
-    dockingColor = defaults.getColor("ToolBar.dockingBackground");
-
-    floatingBorderColor = defaults.getColor("ToolBar.floatingForeground");
-    floatingColor = defaults.getColor("ToolBar.floatingBackground");
+    floatingBorderColor = UIManager.getColor("ToolBar.floatingForeground");
+    floatingColor = UIManager.getColor("ToolBar.floatingBackground");
   }
 
   /**
@@ -585,16 +606,15 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
    * by the look and feel.
    */
   protected void installKeyboardActions()
+    throws NotImplementedException
   {
     // FIXME: implement.
   }
 
   /**
    * This method installs listeners for the JToolBar.
-   *
-   * @param toolbar The JToolBar to register listeners for.
    */
-  protected void installListeners(JToolBar toolbar)
+  protected void installListeners()
   {
     dockingListener = createDockingListener();
     toolBar.addMouseListener(dockingListener);
@@ -691,10 +711,9 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
     if (c instanceof JToolBar)
       {
 	toolBar = (JToolBar) c;
-	toolBar.setOpaque(true);
-	installDefaults();
-	installComponents();
-	installListeners(toolBar);
+    installDefaults();
+    installComponents();
+	installListeners();
 	installKeyboardActions();
       }
   }
@@ -738,9 +757,9 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
    */
   protected void setBorderToNonRollover(Component c)
   {
-    if (c instanceof JButton)
+    if (c instanceof AbstractButton)
       {
-	JButton b = (JButton) c;
+	AbstractButton b = (AbstractButton) c;
 	b.setRolloverEnabled(false);
 	b.setBorder(nonRolloverBorder);
       }
@@ -859,8 +878,6 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
   {
     installNormalBorders(toolBar);
     borders = null;
-    rolloverBorder = null;
-    nonRolloverBorder = null;
     cachedBounds = null;
 
     floatFrame = null;
@@ -886,6 +903,7 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
    * This method uninstalls keyboard actions installed by the UI.
    */
   protected void uninstallKeyboardActions()
+    throws NotImplementedException
   {
     // FIXME: implement.
   }
@@ -1000,6 +1018,7 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
      */
     public void mouseMoved(MouseEvent e)
     {
+      // TODO: What should be done here, if anything?
     }
 
     /**
@@ -1030,18 +1049,20 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
         }
 
       origin = new Point(0, 0);
-      SwingUtilities.convertPointToScreen(ssd, toolBar);
+      if (toolBar.isShowing())
+        SwingUtilities.convertPointToScreen(ssd, toolBar);
 
       if (! (SwingUtilities.getAncestorOfClass(Window.class, toolBar) instanceof UIResource))
 	// Need to know who keeps the toolBar if it gets dragged back into it.
 	origParent = toolBar.getParent();
-
-      SwingUtilities.convertPointToScreen(origin, toolBar);
+      
+      if (toolBar.isShowing())
+        SwingUtilities.convertPointToScreen(origin, toolBar);
 
       isDragging = true;
 
       if (dragWindow != null)
-	dragWindow.setOffset(new Point(e.getX(), e.getY()));
+	dragWindow.setOffset(new Point(cachedBounds.width/2, cachedBounds.height/2));
 
       dragTo(e.getPoint(), origin);
     }
@@ -1212,8 +1233,8 @@ public class BasicToolBarUI extends ToolBarUI implements SwingConstants
     public void propertyChange(PropertyChangeEvent e)
     {
       // FIXME: need name properties so can change floatFrame title.
-      if (e.getPropertyName().equals("rollover"))
-	setRolloverBorders(toolBar.isRollover());
+      if (e.getPropertyName().equals("rollover") && toolBar != null)
+        setRolloverBorders(toolBar.isRollover());
     }
   }
 
