@@ -1,5 +1,5 @@
 /* MetalBorders.java
-   Copyright (C) 2005 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006, Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -56,10 +56,10 @@ import javax.swing.JTextField;
 import javax.swing.JToggleButton;
 import javax.swing.JToolBar;
 import javax.swing.SwingConstants;
-import javax.swing.UIDefaults;
 import javax.swing.UIManager;
 import javax.swing.border.AbstractBorder;
 import javax.swing.border.Border;
+import javax.swing.border.CompoundBorder;
 import javax.swing.plaf.BorderUIResource;
 import javax.swing.plaf.UIResource;
 import javax.swing.plaf.basic.BasicBorders;
@@ -130,6 +130,119 @@ public class MetalBorders
      */
     public void paintBorder(Component c, Graphics g, int x, int y, int w,
                             int h)
+    {
+      // With the OceanTheme the button border is painted entirely different.
+      // However, I couldn't figure out how this is determined besides checking
+      // for instanceof OceanTheme. The button painting is definitely not
+      // influenced by a UI default property and it is definitely performed
+      // by the same Border class.
+      if (MetalLookAndFeel.getCurrentTheme() instanceof OceanTheme)
+        paintOceanButtonBorder(c, g, x, y, w, h);
+      else
+        paintDefaultButtonBorder(c, g, x, y, w, h);
+    }
+
+    /**
+     * Paints the button border for the DefaultMetalTheme.
+     *
+     * @param c the component (button)
+     * @param g the graphics object to use
+     * @param x the upper left corner of the component, X coordinate
+     * @param y the upper left corner of the component, Y coordinate
+     * @param w the width of the component
+     * @param h the height of the component
+     */
+    private void paintDefaultButtonBorder(Component c, Graphics g, int x,
+                                          int y, int w, int h)
+    {
+      ButtonModel bmodel = null;
+
+      if (c instanceof AbstractButton)
+        bmodel = ((AbstractButton) c).getModel();
+
+      Color darkShadow = MetalLookAndFeel.getControlDarkShadow();
+      Color shadow = MetalLookAndFeel.getControlShadow();
+      Color light = MetalLookAndFeel.getControlHighlight();
+      Color middle = MetalLookAndFeel.getControl();
+
+      if (c.isEnabled())
+        {
+          // draw dark border
+          g.setColor(darkShadow);
+          g.drawRect(x, y, w - 2, h - 2);
+
+          // If the button is the default button, we paint a special border,
+          // regardless of the pressed state.
+          if (c instanceof JButton && ((JButton) c).isDefaultButton())
+            {
+              g.drawRect(x + 1, y + 1, w - 4, h - 4);
+              // Draw white highlight.
+              g.setColor(light);
+              g.drawLine(x + 2, y + 2, x + w - 4, y + 2);
+              g.drawLine(x + 2, y + 2, x + 2, y + h - 4);
+              g.drawLine(x + 2, y + h - 1, x + w - 1, y + h - 1);
+              g.drawLine(x + w - 1, y + 2, x + w - 1, y + h - 1);
+              // Draw crossing pixels.
+              g.setColor(middle);
+              g.fillRect(x + w - 2, y + 2, 1, 1);
+              g.fillRect(x + 2, y + h - 2, 1, 1);
+            }
+          else
+            {
+              // The normal border. This is used when the button is not
+              // pressed or the button is not armed.
+              if (! (bmodel.isPressed() && bmodel.isArmed()) )
+                {
+                  // draw light border
+                  g.setColor(light);
+                  g.drawRect(x + 1, y + 1, w - 2, h - 2);
+
+                  // draw crossing pixels of both borders
+                  g.setColor(middle);
+                  g.drawLine(x + 1, y + h - 2, x + 1, y + h - 2);
+                  g.drawLine(x + w - 2, y + 1, x + w - 2, y + 1);
+                }
+              // The pressed border. This border is painted only when
+              // the button is both pressed and armed.
+              else
+                {
+                  // draw light border
+                  g.setColor(light);
+                  g.drawLine(x + w - 1, y + 1, x + w - 1, y + h - 1);
+                  g.drawLine(x + 1, y + h - 1, x + w - 1, y + h - 1);
+
+                  // draw shadow border
+                  g.setColor(middle);
+                  g.drawLine(x + 1, y + 1, x + w - 2, y + 1);
+                  g.drawLine(x + 1, y + 1, x + 1, y + h - 2);
+ 
+                  // draw crossing pixels of both borders
+                  g.setColor(shadow);
+                  g.drawRect(x + 1, y + h - 2, 0, 0);
+                  g.drawRect(x + w - 2, y + 1, 0, 0);
+                }
+            }
+        }
+      else 
+        {
+          // draw disabled border
+          g.setColor(MetalLookAndFeel.getInactiveControlTextColor());
+          g.drawRect(x, y, w - 2, h - 2);          
+        }
+    }
+
+    /**
+     * Paints the button border for the OceanTheme.
+     *
+     * @param c the button
+     * @param g the graphics context
+     * @param x the X coordinate of the upper left corner of the painting rect
+     * @param y the Y coordinate of the upper left corner of the painting rect
+     * @param w the width of the painting rect
+     * @param h the height of the painting rect
+     */
+    private void paintOceanButtonBorder(Component c, Graphics g, int x,
+                                        int y, int w, int h)
     {
       ButtonModel bmodel = null;
       
@@ -138,44 +251,36 @@ public class MetalBorders
 
       Color darkShadow = MetalLookAndFeel.getControlDarkShadow();
       Color shadow = MetalLookAndFeel.getControlShadow();
-      Color light = MetalLookAndFeel.getWhite();
+      Color light = MetalLookAndFeel.getControlHighlight();
       Color middle = MetalLookAndFeel.getControl();
 
       if (c.isEnabled())
-      {
-        // draw dark border
-        g.setColor(darkShadow);
-        g.drawRect(x, y, w - 2, h - 2);
-
-        if (!bmodel.isPressed())
-          {
-            // draw light border
-            g.setColor(light);
-            g.drawRect(x + 1, y + 1, w - 2, h - 2);
-
-            // draw crossing pixels of both borders
-            g.setColor(middle);
-            g.drawRect(x + 1, y + h - 2, 0, 0);
-            g.drawRect(x + w - 2, y + 1, 0, 0);
-          }
-        else
-          {
-            // draw light border
-            g.setColor(light);
-            g.drawLine(x + w - 1, y + 1, x + w - 1, y + h - 1);
-            g.drawLine(x + 1, y + h - 1, x + w - 1, y + h - 1);
-
-            // draw shadow border
-            g.setColor(middle);
-            g.drawLine(x + 1, y + 1, x + w - 2, y + 1);
-            g.drawLine(x + 1, y + 1, x + 1, y + h - 2);
- 
-            // draw crossing pixels of both borders
-            g.setColor(shadow);
-            g.drawRect(x + 1, y + h - 2, 0, 0);
-            g.drawRect(x + w - 2, y + 1, 0, 0);
-          }
-      }
+        {
+          // Paint the pressed border if the button is pressed, or if
+          // the button is the default button. In the OceanTheme, the default
+          // button has the same border as a pressed button.
+          if (bmodel.isPressed() || ((c instanceof JButton)
+                                     && ((JButton) c).isDefaultButton()))
+            {
+              // Draw fat border.
+              g.setColor(darkShadow);
+              g.drawRect(x, y, w - 1, h - 1);
+              g.drawRect(x + 1, y + 1, w - 3, h - 3);
+            }
+          else if (bmodel.isRollover())
+            {
+              g.setColor(shadow);
+              g.drawRect(x, y, w - 1, h - 1);
+              g.drawRect(x + 2, y + 2, w - 5, h - 5);
+              g.setColor(darkShadow);
+              g.drawRect(x + 1, y + 1, w - 3, h - 3);
+            }
+          else
+            {
+              g.setColor(darkShadow);
+              g.drawRect(x, y, w - 1, h - 1);
+            }
+        }
       else 
         {
           // draw disabled border
@@ -187,30 +292,27 @@ public class MetalBorders
     /**
      * Returns the insets of the <code>ButtonBorder</code>.
      *
-     * @param c the component for which the border is used
+     * @param c the component for which the border is used (ignored).
      *
-     * @return The insets of the ButtonBorder
+     * @return The insets of the <code>ButtonBorder</code>.
      */
     public Insets getBorderInsets(Component c)
     {
-      return getBorderInsets(c, null);
+      return borderInsets;
     }
 
     /**
      * Returns the insets of the <code>ButtonBorder</code> in the specified 
      * <code>newInsets</code> object.
      *
-     * @param c the component for which the border is used
-     * @param newInsets the insets object where to put the values (if 
-     *        <code>null</code>, a new instance is created).
+     * @param c the component for which the border is used (ignored).
+     * @param newInsets the insets object where to put the values (
+     *                  <code>null</code> not permitted).
      *
-     * @return The insets.
+     * @return The <code>newInsets</code> reference.
      */
     public Insets getBorderInsets(Component c, Insets newInsets)
     {
-      if (newInsets == null)
-        newInsets = new Insets(0, 0, 0, 0);
-
       newInsets.bottom = borderInsets.bottom;
       newInsets.left = borderInsets.left;
       newInsets.right = borderInsets.right;
@@ -290,6 +392,8 @@ public class MetalBorders
   public static class Flush3DBorder extends AbstractBorder
     implements UIResource
   {
+    private static final Insets borderInsets = new Insets(2, 2, 2, 2);
+    
     /**
      * Creates a new border instance.
      */
@@ -307,26 +411,25 @@ public class MetalBorders
      */
     public Insets getBorderInsets(Component c)
     {
-      return getBorderInsets(c, null);
+      return borderInsets;
     }
     
     /**
      * Returns the border insets.
      * 
      * @param c  the component (ignored).
-     * @return The border insets.
+     * @param newInsets  an existing insets instance, that will be populated
+     *                   with the border insets and returned as the result
+     *                   (<code>null</code> not permitted).
+     *                   
+     * @return The <code>newInsets</code> reference.
      */
     public Insets getBorderInsets(Component c, Insets newInsets)
     {
-      if (newInsets == null)
-        newInsets = new Insets(2, 2, 2, 2);
-      else
-        {
-          newInsets.top = 2;
-          newInsets.left = 2;
-          newInsets.bottom = 2;
-          newInsets.right = 2;
-        }
+      newInsets.top = borderInsets.top;
+      newInsets.left = borderInsets.left;
+      newInsets.bottom = borderInsets.bottom;
+      newInsets.right = borderInsets.right;
       return newInsets;  
     }
     
@@ -365,6 +468,8 @@ public class MetalBorders
   public static class PaletteBorder extends AbstractBorder
     implements UIResource
   {
+    private static final Insets borderInsets = new Insets(1, 1, 1, 1);
+
     /**
      * Creates a new <code>PaletteBorder</code>.
      */
@@ -382,29 +487,25 @@ public class MetalBorders
      */
     public Insets getBorderInsets(Component c)
     {
-      return getBorderInsets(c, null);
+      return borderInsets;
     }
 
     /**
      * Returns the border insets.
      * 
      * @param c  the component (ignored).
-     * @param newInsets  the insets object that, if non-<code>null</code>, will 
-     *                   be populated with the result from this method.
-     * 
-     * @return The border insets.
+     * @param newInsets  an existing insets instance, that will be populated
+     *                   with the border insets and returned as the result
+     *                   (<code>null</code> not permitted).
+     *                   
+     * @return The <code>newInsets</code> reference.
      */
     public Insets getBorderInsets(Component c, Insets newInsets)
     {        
-      if (newInsets == null)
-        newInsets = new Insets(1, 1, 1, 1);
-      else
-        {
-          newInsets.top = 1;
-          newInsets.left = 1;
-          newInsets.bottom = 1;
-          newInsets.right = 1;
-        }
+      newInsets.top = borderInsets.top;
+      newInsets.left = borderInsets.left;
+      newInsets.bottom = borderInsets.bottom;
+      newInsets.right = borderInsets.right;
       return newInsets;  
     }
     
@@ -493,6 +594,8 @@ public class MetalBorders
   public static class InternalFrameBorder extends AbstractBorder
     implements UIResource
   {
+    private static final Insets borderInsets = new Insets(5, 5, 5, 5);
+
     /**
      * Creates a new border instance.
      */
@@ -510,26 +613,25 @@ public class MetalBorders
      */
     public Insets getBorderInsets(Component c)
     {
-      return getBorderInsets(c, null);
+      return borderInsets;
     }
     
     /**
      * Returns the border insets.
      * 
      * @param c  the component (ignored).
-     * @return The border insets.
+     * @param newInsets  an existing insets instance, that will be populated
+     *                   with the border insets and returned as the result
+     *                   (<code>null</code> not permitted).
+     *                   
+     * @return The <code>newInsets</code> reference.
      */
     public Insets getBorderInsets(Component c, Insets newInsets)
     {
-      if (newInsets == null)
-        newInsets = new Insets(5, 5, 5, 5);
-      else
-        {
-          newInsets.top = 5;
-          newInsets.left = 5;
-          newInsets.bottom = 5;
-          newInsets.right = 5;
-        }
+      newInsets.top = borderInsets.top;
+      newInsets.left = borderInsets.left;
+      newInsets.bottom = borderInsets.bottom;
+      newInsets.right = borderInsets.right;
       return newInsets;  
     }
     
@@ -654,24 +756,23 @@ public class MetalBorders
         {
           JOptionPane pane = (JOptionPane) f.getContentPane();
           int type = pane.getMessageType();
-          UIDefaults defaults = UIManager.getLookAndFeelDefaults();
           if (type == JOptionPane.QUESTION_MESSAGE)
             {
-              Color bc = defaults.getColor(
+              Color bc = UIManager.getColor(
                   "OptionPane.questionDialog.border.background");
               if (bc != null)
                 g.setColor(bc);
             }
           if (type == JOptionPane.WARNING_MESSAGE)
             {
-              Color bc = defaults.getColor(
+              Color bc = UIManager.getColor(
                   "OptionPane.warningDialog.border.background");
               if (bc != null)
                 g.setColor(bc);              
             }
           else if (type == JOptionPane.ERROR_MESSAGE)
             {
-              Color bc = defaults.getColor(
+              Color bc = UIManager.getColor(
                   "OptionPane.errorDialog.border.background");
               if (bc != null)
                 g.setColor(bc);              
@@ -702,7 +803,7 @@ public class MetalBorders
     implements UIResource
   {
     /** The border insets. */
-    protected static Insets borderInsets = new Insets(1, 1, 1, 1);
+    protected static Insets borderInsets = new Insets(2, 2, 2, 2);
     
     /**
      * Creates a new border instance.
@@ -967,14 +1068,10 @@ public class MetalBorders
     public void paintBorder(Component c, Graphics g, int x, int y, int w, 
             int h)
     {
-      boolean mouseIsOver = false;
-      if (c instanceof AbstractButton)
-        {
-          ButtonModel bmodel = ((AbstractButton) c).getModel();
-          mouseIsOver = bmodel.isRollover();
-        }
-      if (mouseIsOver)
-        super.paintBorder(c, g, x, y, w, h);
+      // TODO: What should be done here? Obviously the ButtonBorder already
+      // handles the rollover state in Sun's impl. Maybe this is only there
+      // for backwards compatibility.
+      super.paintBorder(c, g, x, y, w, h);
     }
   }
   
@@ -1462,8 +1559,7 @@ public class MetalBorders
       {
         Border outer = new ButtonBorder();
         Border inner = new RolloverMarginBorder();
-        toolbarButtonBorder = new BorderUIResource.CompoundBorderUIResource
-          (outer, inner);
+        toolbarButtonBorder = new CompoundBorder(outer, inner);
       }
     return toolbarButtonBorder;
   }
