@@ -52,6 +52,8 @@ import javax.swing.JInternalFrame;
 import javax.swing.JRadioButton;
 import javax.swing.JRadioButtonMenuItem;
 import javax.swing.JSlider;
+import javax.swing.SwingConstants;
+import javax.swing.UIManager;
 import javax.swing.plaf.UIResource;
 
 
@@ -66,6 +68,12 @@ public class MetalIconFactory implements Serializable
     
   /** A constant representing "light". */
   public static final boolean LIGHT = true;
+  
+  /** A shared instance of the MenuArrowIcon. */
+  private static Icon menuArrow;
+  
+  /** A shared instance of the MenuItemArrowIcon. */
+  private static Icon menuItemArrow;
     
   /**
    * An icon displayed for {@link JCheckBoxMenuItem} components.
@@ -747,6 +755,17 @@ public class MetalIconFactory implements Serializable
   static class RadioButtonIcon implements Icon, UIResource, Serializable
   {
 
+    /**
+     * This is used as a mask when painting the gradient. See
+     * {@link MetalUtils#paintGradient(java.awt.Graphics, int, int, int, int,
+     *  float, float, java.awt.Color, java.awt.Color, java.awt.Color, int,
+     *  int[][])} for details.
+     */
+    private static int[][] gradientMask = new int[][] {{3, 7}, {1, 9}, {1, 9},
+                                                       {0, 10}, {0, 10}, {0, 10},
+                                                       {0, 10}, {1, 9}, {1, 9},
+                                                       {3, 7}};
+
     /**
      * Returns the width of the icon in pixels.
      *
@@ -779,9 +798,14 @@ public class MetalIconFactory implements Serializable
      */
     public void paintIcon(Component c, Graphics g, int x, int y) 
     {
+      if (UIManager.get("RadioButton.gradient") != null)
+        MetalUtils.paintGradient(g, x + 2, y + 2, 8, 8,
+                              SwingConstants.VERTICAL, "RadioButton.gradient",
+                              gradientMask);
+
       Color savedColor = g.getColor();
       JRadioButton b = (JRadioButton) c;
-      
+
       // draw outer circle
       if (b.isEnabled())
         g.setColor(MetalLookAndFeel.getControlDarkShadow());
@@ -938,6 +962,14 @@ public class MetalIconFactory implements Serializable
   private static class HorizontalSliderThumbIcon  implements Icon, Serializable
   {
 
+    /**
+     * This mask is used to paint the gradient in the shape of the thumb.
+     */
+    int[][] gradientMask = new int[][] { {0, 12}, {0, 12}, {0, 12}, {0, 12},
+                                         {0, 12}, {0, 12}, {0, 12}, {1, 12},
+                                         {2, 10}, {3, 9}, {4, 8}, {5, 7},
+                                         {6, 6}};
+
     /**
      * Creates a new instance.
      */
@@ -996,21 +1028,37 @@ public class MetalIconFactory implements Serializable
       g.drawLine(x + 6, y + 14, x, y + 8);
       g.drawLine(x, y + 7, x, y + 1);
       
-      // fill the icon
-      if (focus)
-        g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
+      // Fill the icon.
+      if (MetalLookAndFeel.getCurrentTheme() instanceof OceanTheme
+          && enabled)
+        {
+          String gradient;
+          if (focus)
+            gradient = "Slider.focusGradient";
+          else
+            gradient = "Slider.gradient";
+          MetalUtils.paintGradient(g, x + 1, y + 2, 12, 13,
+                                   SwingConstants.VERTICAL, gradient,
+                                   gradientMask);
+        }
       else
-        g.setColor(MetalLookAndFeel.getControl());
-      g.fillRect(x + 1, y + 2, 13, 7);
-      g.drawLine(x + 2, y + 9, x + 12, y + 9);
-      g.drawLine(x + 3, y + 10, x + 11, y + 10);
-      g.drawLine(x + 4, y + 11, x + 10, y + 11);
-      g.drawLine(x + 5, y + 12, x + 9, y + 12);
-      g.drawLine(x + 6, y + 13, x + 8, y + 13);
-      g.drawLine(x + 7, y + 14, x + 7, y + 14);
-      
-      // if the slider is enabled, draw dots and highlights
-      if (c.isEnabled())
+        {
+          if (focus)
+            g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
+          else
+            g.setColor(MetalLookAndFeel.getControl());
+          g.fillRect(x + 1, y + 2, 13, 7);
+          g.drawLine(x + 2, y + 9, x + 12, y + 9);
+          g.drawLine(x + 3, y + 10, x + 11, y + 10);
+          g.drawLine(x + 4, y + 11, x + 10, y + 11);
+          g.drawLine(x + 5, y + 12, x + 9, y + 12);
+          g.drawLine(x + 6, y + 13, x + 8, y + 13);
+          g.drawLine(x + 7, y + 14, x + 7, y + 14);
+        }
+
+      // If the slider is enabled, draw dots and highlights.
+      if (c.isEnabled()
+          && !(MetalLookAndFeel.getCurrentTheme() instanceof OceanTheme))
         {
           if (focus)
             g.setColor(MetalLookAndFeel.getPrimaryControlDarkShadow());
@@ -1027,7 +1075,7 @@ public class MetalIconFactory implements Serializable
           g.drawLine(x + 7, y + 7, x + 7, y + 7);
           g.drawLine(x + 11, y + 7, x + 11, y + 7);
 
-          // draw highlights
+          // Draw highlights
           if (focus)
             g.setColor(MetalLookAndFeel.getPrimaryControl());
           else
@@ -1570,6 +1618,14 @@ public class MetalIconFactory implements Serializable
    */
   private static class VerticalSliderThumbIcon implements Icon, Serializable
   {
+    /**
+     * This mask is used to paint the gradient in the shape of the thumb.
+     */
+    int[][] gradientMask = new int[][] { {0, 12}, {0, 12}, {0, 12}, {0, 12},
+                                         {0, 12}, {0, 12}, {0, 12}, {1, 12},
+                                         {2, 10}, {3, 9}, {4, 8}, {5, 7},
+                                         {6, 6}};
+
     /**
      * Creates a new instance.
      */
@@ -1629,21 +1685,37 @@ public class MetalIconFactory implements Serializable
       g.drawLine(x + 8, y + 14, x + 1, y + 14);
       g.drawLine(x, y + 13, x, y + 1);
       
-      // fill the icon
-      if (focus)
-        g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
+      // Fill the icon.
+      if (MetalLookAndFeel.getCurrentTheme() instanceof OceanTheme
+          && enabled)
+        {
+          String gradient;
+          if (focus)
+            gradient = "Slider.focusGradient";
+          else
+            gradient = "Slider.gradient";
+          MetalUtils.paintGradient(g, x + 2, y + 1, 13, 12,
+                                   SwingConstants.HORIZONTAL, gradient,
+                                   gradientMask);
+        }
       else
-        g.setColor(MetalLookAndFeel.getControl());
-      g.fillRect(x + 2, y + 1, 7, 13);
-      g.drawLine(x + 9, y + 2, x + 9, y + 12);
-      g.drawLine(x + 10, y + 3, x + 10, y + 11);
-      g.drawLine(x + 11, y + 4, x + 11, y + 10);
-      g.drawLine(x + 12, y + 5, x + 12, y + 9);
-      g.drawLine(x + 13, y + 6, x + 13, y + 8);
-      g.drawLine(x + 14, y + 7, x + 14, y + 7);
-      
+        {
+          if (focus)
+            g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
+          else
+            g.setColor(MetalLookAndFeel.getControl());
+          g.fillRect(x + 2, y + 1, 7, 13);
+          g.drawLine(x + 9, y + 2, x + 9, y + 12);
+          g.drawLine(x + 10, y + 3, x + 10, y + 11);
+          g.drawLine(x + 11, y + 4, x + 11, y + 10);
+          g.drawLine(x + 12, y + 5, x + 12, y + 9);
+          g.drawLine(x + 13, y + 6, x + 13, y + 8);
+          g.drawLine(x + 14, y + 7, x + 14, y + 7);
+        }
+
       // if the slider is enabled, draw dots and highlights
-      if (enabled)
+      if (enabled
+          && ! (MetalLookAndFeel.getCurrentTheme() instanceof OceanTheme))
         {
           if (focus)
             g.setColor(MetalLookAndFeel.getPrimaryControlDarkShadow());
@@ -2461,4 +2533,102 @@ public class MetalIconFactory implements Serializable
     return treeHardDriveIcon;
   }
 
+  /**
+   * Returns a new instance of a 4 x 8 icon showing a small black triangle that
+   * points to the right.  This is displayed in menu items that have a 
+   * sub menu.
+   * 
+   * @return The icon.
+   */
+  public static Icon getMenuArrowIcon()
+  {
+    if (menuArrow == null)
+      menuArrow = new Icon()
+      {
+        public int getIconHeight()
+        {
+          return 8;
+        }
+
+        public int getIconWidth()
+        {
+          return 4;
+        }
+
+        public void paintIcon(Component c, Graphics g, int x, int y)
+        {
+          Color saved = g.getColor();
+          g.setColor(Color.BLACK);
+          for (int i = 0; i < 4; i++)
+            g.drawLine(x + i, y + i, x + i, y + 7 - i);
+          g.setColor(saved);
+        }
+      };
+    return menuArrow;
+  }
+  
+  /**
+   * Returns a new instance of a 4 x 8 icon showing a small black triangle that
+   * points to the right. This is displayed in menu items that have a sub menu.
+   * 
+   * @return The icon.
+   */
+  public static Icon getMenuItemArrowIcon()
+  {
+    if (menuItemArrow == null)
+      menuItemArrow = new Icon()
+      {
+        public int getIconHeight()
+        {
+          return 8;
+        }
+
+        public int getIconWidth()
+        {
+          return 4;
+        }
+
+        public void paintIcon(Component c, Graphics g, int x, int y)
+        {
+          Color saved = g.getColor();
+          g.setColor(Color.BLACK);
+          for (int i = 0; i < 4; i++)
+            g.drawLine(x + i, y + i, x + i, y + 7 - i);
+          g.setColor(saved);
+        }
+      };
+    return menuItemArrow;
+  }
+  
+  /**
+   * Returns a new instance of a 13 x 13 icon showing a small black check mark.
+   * 
+   * @return The icon.
+   */
+  public static Icon getMenuItemCheckIcon()
+  {
+    return new Icon()
+    {
+      public int getIconHeight()
+      {
+        return 13;
+      }
+
+      public int getIconWidth()
+      {
+        return 13;
+      }
+
+      public void paintIcon(Component c, Graphics g, int x, int y)
+      {
+        Color saved = g.getColor();
+        g.setColor(Color.BLACK);
+        g.drawLine(3 + x, 5 + y, 3 + x, 9 + y);
+        g.drawLine(4 + x, 5 + y, 4 + x, 9 + y);
+        g.drawLine(5 + x, 7 + y, 9 + x, 3 + y);
+        g.drawLine(5 + x, 8 + y, 9 + x, 4 + y);
+        g.setColor(saved);
+      }
+    };
+  }
 }
