@@ -294,12 +294,7 @@ public class DefaultTableModel extends AbstractTableModel
     else 
     {
       int rowsToAdd = rowCount - existingRowCount;
-      for (int i = 0; i < rowsToAdd; i++) 
-        {
-          Vector tmp = new Vector();
-          tmp.setSize(columnIdentifiers.size());
-          dataVector.add(tmp);
-        } 
+      addExtraRows(rowsToAdd, columnIdentifiers.size());
       fireTableRowsInserted(existingRowCount,rowCount-1);
     }
   }
@@ -366,12 +361,7 @@ public class DefaultTableModel extends AbstractTableModel
       if (columnData.length > dataVector.size()) 
       {
         int rowsToAdd = columnData.length - dataVector.size();
-        for (int i = 0; i < rowsToAdd; i++) 
-        {
-          Vector tmp = new Vector();
-          tmp.setSize(columnIdentifiers.size());
-          dataVector.add(tmp);
-        }
+        addExtraRows(rowsToAdd, columnIdentifiers.size());
       }
       else if (columnData.length < dataVector.size())
       {
@@ -486,7 +476,10 @@ public class DefaultTableModel extends AbstractTableModel
   }
 
   /**
-   * Returns the name of the specified column.
+   * Get the name of the column. If the column has the column identifier set,
+   * the return value is the result of the .toString() method call on that
+   * identifier. If the identifier is not explicitly set, the returned value
+   * is calculated by {@link AbstractTableModel#getColumnName(int)}.
    * 
    * @param column the column index.
    * 
@@ -499,7 +492,8 @@ public class DefaultTableModel extends AbstractTableModel
     else 
     {
       if (column < getColumnCount())
-      {  
+      {
+        checkSize();
         Object id = columnIdentifiers.get(column);
         if (id != null) 
           result = id.toString();
@@ -585,4 +579,41 @@ public class DefaultTableModel extends AbstractTableModel
       vector.add(convertToVector(data[i]));
     return vector;
   }
+
+  /**
+   * This method adds some rows to <code>dataVector</code>.
+   *
+   * @param rowsToAdd number of rows to add
+   * @param nbColumns size of the added rows
+   */
+  private void addExtraRows(int rowsToAdd, int nbColumns)
+  {
+    for (int i = 0; i < rowsToAdd; i++) 
+      {
+        Vector tmp = new Vector();
+        tmp.setSize(columnIdentifiers.size());
+        dataVector.add(tmp);
+      } 
+  }
+
+  /**
+   * Checks the real columns/rows sizes against the ones returned by
+   * <code>getColumnCount()</code> and <code>getRowCount()</code>.
+   * If the supposed one are bigger, then we grow <code>columIdentifiers</code>
+   * and <code>dataVector</code> to their expected size.
+   */
+  private void checkSize()
+  {
+    int columnCount = getColumnCount();
+    int rowCount = getRowCount();
+    
+    if (columnCount > columnIdentifiers.size())
+      columnIdentifiers.setSize(columnCount);
+           
+    if (rowCount > dataVector.size())
+      {
+        int rowsToAdd = rowCount - dataVector.size();
+        addExtraRows(rowsToAdd, columnCount);
+      }
+  }
 }
