@@ -51,7 +51,6 @@ import javax.swing.border.Border;
 import javax.swing.Icon;
 import javax.swing.JLabel;
 import javax.swing.JTree;
-import javax.swing.UIDefaults;
 import javax.swing.UIManager;
 import javax.swing.SwingUtilities;
 import javax.swing.plaf.UIResource;
@@ -133,17 +132,15 @@ public class DefaultTreeCellRenderer
    */
   public DefaultTreeCellRenderer()
   {
-    UIDefaults defaults = UIManager.getLookAndFeelDefaults();
-
     setLeafIcon(getDefaultLeafIcon());
     setOpenIcon(getDefaultOpenIcon());
     setClosedIcon(getDefaultClosedIcon());
 
-    setTextNonSelectionColor(defaults.getColor("Tree.textForeground"));
-    setTextSelectionColor(defaults.getColor("Tree.selectionForeground"));
-    setBackgroundNonSelectionColor(defaults.getColor("Tree.nonSelectionBackground"));
-    setBackgroundSelectionColor(defaults.getColor("Tree.selectionBackground"));
-    setBorderSelectionColor(defaults.getColor("Tree.selectionBorderColor"));
+    setTextNonSelectionColor(UIManager.getColor("Tree.textForeground"));
+    setTextSelectionColor(UIManager.getColor("Tree.selectionForeground"));
+    setBackgroundNonSelectionColor(UIManager.getColor("Tree.nonSelectionBackground"));
+    setBackgroundSelectionColor(UIManager.getColor("Tree.selectionBackground"));
+    setBorderSelectionColor(UIManager.getColor("Tree.selectionBorderColor"));
   }
 
   // -------------------------------------------------------------
@@ -157,7 +154,7 @@ public class DefaultTreeCellRenderer
    */
   public Icon getDefaultOpenIcon()
   {
-    return UIManager.getLookAndFeelDefaults().getIcon("Tree.openIcon");
+    return UIManager.getIcon("Tree.openIcon");
   }
 
   /**
@@ -167,7 +164,7 @@ public class DefaultTreeCellRenderer
    */
   public Icon getDefaultClosedIcon()
   {
-    return UIManager.getLookAndFeelDefaults().getIcon("Tree.closedIcon");
+    return UIManager.getIcon("Tree.closedIcon");
   }
 
   /**
@@ -177,7 +174,7 @@ public class DefaultTreeCellRenderer
    */
   public Icon getDefaultLeafIcon()
   {
-    return UIManager.getLookAndFeelDefaults().getIcon("Tree.leafIcon");
+    return UIManager.getIcon("Tree.leafIcon");
   }
 
   /**
@@ -410,17 +407,16 @@ public class DefaultTreeCellRenderer
     this.hasFocus = hasFocus;
     setHorizontalAlignment(LEFT);
     setOpaque(false);
-    setVerticalAlignment(TOP);
+    setVerticalAlignment(CENTER);
     setEnabled(true);
-    super.setFont(UIManager.getLookAndFeelDefaults().getFont("Tree.font"));
+    super.setFont(UIManager.getFont("Tree.font"));
 
     if (selected)
       {
         super.setBackground(getBackgroundSelectionColor());
         setForeground(getTextSelectionColor());
         
-        if (tree.getLeadSelectionPath() == null || 
-            (tree.getLeadSelectionPath().getLastPathComponent()).equals(val))
+        if (hasFocus)
           setBorderSelectionColor(UIManager.getLookAndFeelDefaults().
                                   getColor("Tree.selectionBorderColor"));
         else
@@ -449,8 +445,7 @@ public class DefaultTreeCellRenderer
   /**
    * Paints the value. The background is filled based on selected.
    * 
-   * @param g
-   *          the graphics device.
+   * @param g the graphics device.
    */
   public void paint(Graphics g)
   {
@@ -460,8 +455,7 @@ public class DefaultTreeCellRenderer
     Rectangle tr = new Rectangle();
 
     Insets insets = new Insets(0, 0, 0, 0);
-    Border border = UIManager.getLookAndFeelDefaults().getBorder(
-                                                                 "Tree.selectionBorder");
+    Border border = UIManager.getBorder("Tree.selectionBorder");
     if (border != null)
       insets = border.getBorderInsets(this);
 
@@ -473,17 +467,27 @@ public class DefaultTreeCellRenderer
                                        getHorizontalTextPosition(), vr, ir, tr,
                                        getIconTextGap());
 
+    // Reusing one rectangle.
+    Rectangle bounds = getBounds(ir);
+    
+    bounds.x = tr.x - insets.left;
+    bounds.width = tr.width + insets.left+insets.right;
+    
     g.setColor(super.getBackground());
-    g.fillRect(tr.x, tr.y, tr.width, tr.height - insets.top - insets.bottom);
+    g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
 
-    // paint border
-    Color b = getBorderSelectionColor();
-    if (b != null)
-      {
-        g.setColor(b);
-        g.drawRect(tr.x, tr.y, tr.width, tr.height - insets.top - insets.bottom);
-      }
     super.paint(g);
+    
+    // Paint the border of the focused element only (lead selection)
+    if (hasFocus)
+      {
+        Color b = getBorderSelectionColor();
+        if (b != null)
+          {
+            g.setColor(b);
+            g.drawRect(bounds.x, bounds.y, bounds.width, bounds.height - 1);
+          }
+      }
   }
 
   /**
