@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -49,7 +49,7 @@ exception statement from your version. */
  */
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GThreadNativeMethodRunner_nativeRun
-  (JNIEnv *gdk_env __attribute__((unused)),
+  (JNIEnv *env __attribute__((unused)),
    jobject lcl_obj __attribute__((unused)),
    jlong funcAddr, jlong funcArg)
 {
