@@ -43,6 +43,46 @@ exception statement from your version. */
 
 static jmethodID initComponentGraphicsUnlockedID;
 
+/*
+ * AWT applications may call Graphics methods from threads other than
+ * the GDK main thread, so we must call XFlush after each batch of
+ * drawing operations, otherwise animations flicker.  Flushing after
+ * every graphics operation is excessive and negatively affects
+ * performance (PR 26486).  We set the maximum frequency to 50 times
+ * per second, or a minimum period of 20 milliseconds between calls to
+ * XFlush.  See gnu.classpath.examples.awt.AnimationApplet for an
+ * example applet that requires these XFlush calls.
+ */
+
+static short flush_scheduled = 0;
+
+static gboolean flush (gpointer data __attribute__((unused)))
+{
+  gdk_threads_enter ();
+
+  XFlush (GDK_DISPLAY ());
+  flush_scheduled = 0;
+
+  gdk_threads_leave ();
+
+  return FALSE;
+}
+
+/* The minimum time period between calls to XFlush, in
+   milliseconds. */
+#define MINIMUM_FLUSH_PERIOD 20
+
+/* schedule_flush must be called with the GDK lock held. */
+static void
+schedule_flush ()
+{
+  if (!flush_scheduled)
+    {
+      g_timeout_add (MINIMUM_FLUSH_PERIOD, flush, NULL);
+      flush_scheduled = 1;
+    }
+}
+
 void
 cp_gtk_graphics_init_jni (void)
 {
@@ -99,7 +139,7 @@ static GdkPoint *translate_points (JNIEnv *env, jintArray xpoints,
 static void realize_cb (GtkWidget *widget, jobject jgraphics);
 
 JNIEXPORT void JNICALL
-Java_gnu_java_awt_peer_gtk_GdkGraphics_copyState
+Java_gnu_java_awt_peer_gtk_GdkGraphics_nativeCopyState
   (JNIEnv *env, jobject obj, jobject old)
 {
   struct graphics *g = NULL;
@@ -120,7 +160,8 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_copyState
   else /* GDK_IS_WINDOW (g->drawable) */
     g_object_ref (g->drawable);
 
-  g_object_ref (g->cm);
+  if (g->cm != NULL)
+    g_object_ref (g->cm);
 
   NSA_SET_G_PTR (env, obj, g);
 
@@ -141,7 +182,9 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_initState__II
   g->drawable = (GdkDrawable *) gdk_pixmap_new (NULL, width, height, 
 						gdk_rgb_get_visual ()->depth);
   g->cm = gdk_rgb_get_colormap ();
-  g_object_ref (g->cm);
+  
+  if (g->cm != NULL)
+    g_object_ref (g->cm);
   g->gc = gdk_gc_new (g->drawable);
 
   NSA_SET_G_PTR (env, obj, g);
@@ -168,7 +211,9 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_initFromImage
   g->drawable = (GdkDrawable *)pixmap;
 
   g->cm = gdk_drawable_get_colormap (g->drawable);
-  g_object_ref (g->cm);
+  
+  if (g->cm != NULL)
+    g_object_ref (g->cm);
   g->gc = gdk_gc_new (g->drawable);
 
   NSA_SET_G_PTR (env, obj, g);
@@ -195,7 +240,10 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_initStateUnlocked
 
   g_object_ref (g->drawable);
   g->cm = gtk_widget_get_colormap (widget);
-  g_object_ref (g->cm);
+  
+  if (g->cm != NULL)
+    g_object_ref (g->cm);
+    
   g->gc = gdk_gc_new (g->drawable);
   gdk_gc_copy (g->gc, widget->style->fg_gc[GTK_STATE_NORMAL]);
   color = widget->style->fg[GTK_STATE_NORMAL];
@@ -236,7 +284,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_connectSignals
 }
 
 JNIEXPORT void JNICALL
-Java_gnu_java_awt_peer_gtk_GdkGraphics_dispose
+Java_gnu_java_awt_peer_gtk_GdkGraphics_nativeDispose
   (JNIEnv *env, jobject obj)
 {
   struct graphics *g = NULL;
@@ -253,15 +301,17 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_dispose
     }
 
   XFlush (GDK_DISPLAY ());
-
-  g_object_unref (g->gc);
+  
+  if (g->gc != NULL)
+    g_object_unref (g->gc);
 
   if (GDK_STABLE_IS_PIXMAP (g->drawable))
     g_object_unref (g->drawable);
-  else /* GDK_IS_WINDOW (g->drawable) */
+  else if (g->drawable != NULL)
     g_object_unref (g->drawable);
 
-  g_object_unref (g->cm);
+  if (g->cm != NULL)
+    g_object_unref (g->cm);
 
   g_free (g);
 
@@ -291,6 +341,11 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawString
   struct peerfont *pfont = NULL;
   struct graphics *g = NULL;
   const char *cstr = NULL;
+  const char *sTmp = NULL;
+  char *tmp = NULL;
+  char *p = NULL;
+  int count = 0;
+  int charSize = 0;
   int baseline_y = 0;
   PangoLayoutIter *iter = NULL;
 
@@ -303,9 +358,29 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawString
   g_assert (pfont != NULL);
 
   cstr = (*env)->GetStringUTFChars (env, str, NULL);
+  g_assert (cstr != NULL);  
+  
+  charSize = sizeof(char);
+  p = malloc((strlen(cstr) + 1) * charSize);
+  g_assert (p != NULL);  
+
+  tmp = p;
+  sTmp = cstr;
+  for (; *sTmp != '\0'; sTmp++)
+    if (((unsigned char) *sTmp) >= ' ')
+      {
+        *p = *sTmp;
+        count++;
+        p++;
+      }
+  *p = '\0';
+
+  p = realloc(tmp, (count + 1) * charSize);
+  g_assert (p != NULL);
+  pango_layout_set_text (pfont->layout, p, -1);
+  free(p);
 
   pango_layout_set_font_description (pfont->layout, pfont->desc);
-  pango_layout_set_text (pfont->layout, cstr, -1);
   iter = pango_layout_get_iter (pfont->layout);
 
   baseline_y = pango_layout_iter_get_baseline (iter);
@@ -317,8 +392,8 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawString
 
   pango_layout_iter_free (iter);
   pango_layout_set_text (pfont->layout, "", -1);
-
-  gdk_flush ();
+  
+  schedule_flush ();
 
   (*env)->ReleaseStringUTFChars (env, str, cstr);
 
@@ -338,7 +413,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawLine
   gdk_draw_line (g->drawable, g->gc, 
 		 x + g->x_offset, y + g->y_offset, 
 		 x2 + g->x_offset, y2 + g->y_offset);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }
@@ -355,7 +430,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_fillRect
 
   gdk_draw_rectangle (g->drawable, g->gc, TRUE, 
 		      x + g->x_offset, y + g->y_offset, width, height);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }
@@ -372,7 +447,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawRect
 
   gdk_draw_rectangle (g->drawable, g->gc, FALSE, 
 		      x + g->x_offset, y + g->y_offset, width, height);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }
@@ -394,7 +469,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_copyArea
 		     x + g->x_offset, y + g->y_offset,
 		     x + g->x_offset + dx, y + g->y_offset + dy,
 		     width, height);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }
@@ -430,13 +505,13 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_clearRect
   else
     {
       gdk_gc_get_values (g->gc, &saved);
-      gdk_gc_set_foreground (g->gc, &(saved.background));
+      gdk_gc_set_background (g->gc, &(saved.background));
       gdk_draw_rectangle (g->drawable, g->gc, TRUE, 
 			  x + g->x_offset, y + g->y_offset, width, height);
       gdk_gc_set_foreground (g->gc, &(saved.foreground));
     }
 
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }
@@ -471,10 +546,12 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_setFGColor
   color.blue = blue << 8;
 
   g = (struct graphics *) NSA_GET_G_PTR (env, obj);
-
-  gdk_colormap_alloc_color (g->cm, &color, TRUE, TRUE);
+  
+  if (g->cm != NULL)
+    gdk_colormap_alloc_color (g->cm, &color, TRUE, TRUE);
+    
   gdk_gc_set_foreground (g->gc, &color);
-
+  
   gdk_threads_leave ();
 }
 
@@ -492,7 +569,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawArc
   gdk_draw_arc (g->drawable, g->gc, FALSE, 
 		x + g->x_offset, y + g->y_offset, 
 		width, height, angle1 << 6, angle2 << 6);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }  
@@ -539,7 +616,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawPolyline
 			     g->x_offset, g->y_offset);
 
   gdk_draw_lines (g->drawable, g->gc, points, npoints);
-  gdk_flush ();
+  schedule_flush ();
 
   g_free (points);
 
@@ -566,7 +643,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawPolygon
     points[npoints++] = points[0];
 
   gdk_draw_lines (g->drawable, g->gc, points, npoints);
-  gdk_flush ();
+  schedule_flush ();
 
   g_free (points);
 
@@ -587,7 +664,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_fillPolygon
   points = translate_points (env, xpoints, ypoints, npoints,
 			     g->x_offset, g->y_offset);
   gdk_draw_polygon (g->drawable, g->gc, TRUE, points, npoints);
-  gdk_flush ();
+  schedule_flush ();
 
   g_free (points);
 
@@ -608,7 +685,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_fillArc
   gdk_draw_arc (g->drawable, g->gc, TRUE, 
 		x + g->x_offset, y + g->y_offset, 
 		width, height, angle1 << 6, angle2 << 6);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }  
@@ -626,7 +703,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_drawOval
   gdk_draw_arc (g->drawable, g->gc, FALSE, 
 		x + g->x_offset, y + g->y_offset, 
 		width, height, 0, 23040);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }  
@@ -644,7 +721,7 @@ Java_gnu_java_awt_peer_gtk_GdkGraphics_fillOval
   gdk_draw_arc (g->drawable, g->gc, TRUE, 
 		x + g->x_offset, y + g->y_offset, 
 		width, height, 0, 23040);
-  gdk_flush ();
+  schedule_flush ();
 
   gdk_threads_leave ();
 }
